@@ -14,18 +14,18 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 
 package Torrus::ACL::Edit;
+use strict;
+use warnings;
+
+use base 'Torrus::ACL';
 
 use Torrus::ACL;
 use Torrus::Log;
 
-use strict;
-
-@Torrus::ACL::Edit::ISA = qw(Torrus::ACL);
 
 sub new
 {
@@ -121,7 +121,7 @@ sub listGroups
 
     my $list = $self->{'db_users'}->get( 'G:' );
 
-    return split( ',', $list );
+    return (defined($list) ? split( ',', $list ) : ());
 }
 
 
@@ -336,6 +336,7 @@ sub delUserAttribute
         $self->setUserModified( $uid );
         Debug('Deleted ' . $attr . ' from ' . $uid);
     }
+    return;
 }
 
 
@@ -363,6 +364,7 @@ sub setUserModified
     my $uid = shift;
 
     $self->setUserAttribute( $uid, 'modified', scalar( localtime( time() ) ) );
+    return;
 }
 
 sub listUserAttributes
@@ -372,7 +374,7 @@ sub listUserAttributes
 
     my $list = $self->{'db_users'}->get( 'uA:' . $uid );
 
-    return split( ',', $list );
+    return (defined($list) ? split( ',', $list ) : ());
 }
 
 
@@ -469,10 +471,9 @@ sub listGroupAttributes
 {
     my $self = shift;
     my $group = shift;
-
+    
     my $list = $self->{'db_users'}->get( 'gA:' . $group );
-
-    return split( ',', $list );
+    return (defined($list) ? split( ',', $list ) : ());
 }
 
 
@@ -484,6 +485,7 @@ sub setGroupModified
 
     $self->setGroupAttribute( $group, 'modified',
                               scalar( localtime( time() ) ) );
+    return;
 }
 
 
@@ -579,11 +581,11 @@ sub exportACL
     my $self = shift;
     my $exportfile = shift;
     my $exporttemplate = shift;
-
-    my $ok;
-    eval 'require Torrus::ACL::Export;
-          $ok = Torrus::ACL::Export::exportACL( $self, $exportfile,
-                                              $exporttemplate );';
+    
+    my $ok = 
+        eval( 'require Torrus::ACL::Export;' . 
+              'Torrus::ACL::Export::exportACL($self, $exportfile,'.
+              '$exporttemplate)' );
     if( $@ )
     {
         Error($@);
@@ -600,10 +602,10 @@ sub importACL
     my $self = shift;
     my $importfile = shift;
 
-    my $ok;
-    eval 'require Torrus::ACL::Import;
-          $ok = Torrus::ACL::Import::importACL( $self, $importfile );';
-
+    my $ok =
+        eval('require Torrus::ACL::Import;' . 
+             'Torrus::ACL::Import::importACL($self, $importfile)');
+    
     if( $@ )
     {
         Error($@);
