@@ -14,18 +14,18 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 
 package Torrus::ACL::Import;
+use strict;
+use warnings;
 
 use Torrus::ACL;
 use Torrus::ACL::Edit;
 use Torrus::Log;
 
 use XML::LibXML;
-use strict;
 
 my %formatsSupported = ('1.0' => 1,
                         '1.1' => 1);
@@ -37,9 +37,8 @@ sub importACL
 
     my $ok = 1;
     my $parser = new XML::LibXML;
-    my $doc;
-    eval { $doc = $parser->parse_file( $filename );  };
-    if( $@ )
+    my $doc;    
+    if(not eval {$doc = $parser->parse_file( $filename )} or $@ )
     {
         Error("Failed to parse $filename: $@");
         return 0;
