@@ -589,9 +589,7 @@ sub lookupMap
             (scalar(@mappingSessions) >=
              $Torrus::Collector::SNMP::maxSessionsPerDispatcher) )
         {
-            snmp_dispatcher();
-            @mappingSessions = ();
-            %mapLookupScheduled = ();
+            start_snmp_dispatcher();
         }
 
         # Retrieve map from host
@@ -609,11 +607,20 @@ sub lookupMap
 
         # Retrieve the map table
 
-        my $maxrepetitions = $collector->param($token, 'snmp-maxrepetitions');
-        $session->get_table( -baseoid => $map,
-                             -maxrepetitions => $maxrepetitions,
-                             -callback => [\&mapLookupCallback,
-                                           $collector, $hosthash, $map] );
+        {
+            my @arg = ( -baseoid => $map,
+                        -callback => [\&mapLookupCallback,
+                                      $collector, $hosthash, $map] );
+            
+            if( $session->version() > 0 )
+            {
+                my $maxrepetitions =
+                    $collector->param($token, 'snmp-maxrepetitions');
+                push( @arg, '-maxrepetitions',  $maxrepetitions );
+            }
+            
+            $session->get_table( @arg );
+        }
 
         $mapLookupScheduled{$maphash} = 1;
 
@@ -698,9 +705,19 @@ sub mapLookupCallback
 }
 
 
-sub activeMappingSessions
+
+sub isMapReady
 {
-    return scalar( @mappingSessions );
+    my $hosthash = shift;
+    my $map = shift;
+
+    if( defined($maps{$hosthash}{$map}) and
+        scalar(%{$maps{$hosthash}{$map}}) )
+    {
+        return 1;
+    }
+
+    return 0;
 }
 
 
@@ -1020,21 +1037,28 @@ sub runCollector
         }
         
         &Torrus::DB::checkInterrupted();
-        
-        snmp_dispatcher();
-
-        # Check if there were pending map lookup sessions
-        
-        if( scalar( @mappingSessions ) > 0 )
-        {
-            @mappingSessions = ();
-            %mapLookupScheduled = ();
-        }
+        start_snmp_dispatcher();
     }
     return;
 }
 
 
+
+sub start_snmp_dispatcher
+{
+    snmp_dispatcher();
+    
+    # Check if there were pending map lookup sessions
+    
+    if( scalar( @mappingSessions ) > 0 )
+    {
+        @mappingSessions = ();
+        %mapLookupScheduled = ();
+    }
+}
+
+
+
 sub callback
 {
     my $session = shift;
