@@ -14,18 +14,18 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 
 package Torrus::ConfigTree;
 
+use strict;
+use warnings;
+
 use Torrus::DB;
 use Torrus::Log;
 use Torrus::TimeStamp;
 
-use strict;
-
 
 
 sub new
@@ -267,12 +267,13 @@ sub DESTROY
         $self->clearReader();
     }
 
-    undef $self->{'db_dsconfig'};
-    undef $self->{'db_otherconfig'};
-    undef $self->{'db_aliases'};
-    undef $self->{'db_sets'};
-    undef $self->{'db_nodepcache'};
-    undef $self->{'db_readers'};
+    delete $self->{'db_dsconfig'};
+    delete $self->{'db_otherconfig'};
+    delete $self->{'db_aliases'};
+    delete $self->{'db_sets'};
+    delete $self->{'db_nodepcache'};
+    delete $self->{'db_readers'};
+    return;
 }
 
 # Manage the readinness flag
@@ -282,6 +283,7 @@ sub setReady
     my $self = shift;
     my $ready = shift;
     $self->{'db_otherconfig'}->put( 'ConfigurationReady', $ready ? 1:0 );
+    return;
 }
 
 sub isReady
@@ -304,6 +306,7 @@ sub setReader
                                         time(),
                                         $self->{'ds_config_instance'},
                                         $self->{'other_config_instance'}) );
+    return;
 }
 
 sub clearReader
@@ -316,6 +319,7 @@ sub clearReader
         $self->{'db_readers'}->del( $self->{'reader_id'} );
         delete $self->{'reader_id'};
     }
+    return;
 }
 
 
@@ -328,19 +332,23 @@ sub waitReaders
     while( not $noReaders )
     {
         my @readers = ();
-        my $cursor = $self->{'db_readers'}->cursor();
-        while( my ($key, $val) = $self->{'db_readers'}->next( $cursor ) )
+        
         {
-            my( $timestamp, $dsInst, $otherInst ) = split( /:/o, $val );
-            if( $dsInst == $self->{'ds_config_instance'} or
-                $otherInst == $self->{'other_config_instance'} )
+            my $cursor = $self->{'db_readers'}->cursor();
+            while( my ($key, $val) = $self->{'db_readers'}->next( $cursor ) )
             {
-                push( @readers, {
-                    'reader' => $key,
-                    'timestamp' => $timestamp } );
+                my( $timestamp, $dsInst, $otherInst ) = split( /:/o, $val );
+                if( $dsInst == $self->{'ds_config_instance'} or
+                    $otherInst == $self->{'other_config_instance'} )
+                {
+                    push( @readers, {
+                        'reader' => $key,
+                        'timestamp' => $timestamp } );
+                }
             }
+            $self->{'db_readers'}->c_close($cursor);
         }
-        $self->{'db_readers'}->c_close($cursor);
+        
         if( @readers > 0 )
         {
             Info('Waiting for ' . scalar(@readers) . ' readers:');
@@ -390,6 +398,7 @@ sub waitReaders
             $noReaders = 1;
         }
     }
+    return;
 }
 
 
@@ -419,8 +428,16 @@ sub splitPath
     while( length($path) > 0 )
     {
         my $node;
-        $path =~ s/^([^\/]*\/?)//o; $node = $1;
-        push(@ret, $node);
+        $path =~ s/^([^\/]*\/?)//o;
+        if( defined($1) )
+        {
+            $node = $1;
+            push(@ret, $node);
+        }
+        else
+        {
+            last;
+        }
     }
     return @ret;
 }
@@ -888,6 +905,13 @@ sub getRelative
     my $token = shift;
     my $relPath = shift;
 
+    if( $relPath =~ s/^\[\[(.+)\]\]//o )
+    {
+        my $nodeid = $1;
+        $token = $self->getNodeByNodeid( $nodeid );
+        return(undef) unless defined($token);        
+    }
+    
     if( $relPath =~ /^\//o )
     {
         return $self->token( $relPath );
@@ -912,7 +936,15 @@ sub getRelative
             else
             {
                 my $childName;
-                $relPath =~ s/^([^\/]*\/?)//o; $childName = $1;
+                $relPath =~ s/^([^\/]*\/?)//o;
+                if( defined($1) )
+                {
+                    $childName = $1;
+                }
+                else
+                {
+                    last;
+                }
                 my $path = $self->path( $token );
                 $token = $self->token( $path . $childName );
                 if( not defined $token )
@@ -1082,6 +1114,7 @@ sub addTset
     my $self = shift;
     my $tset = shift;
     $self->{'db_sets'}->addToList('S:', $tset);
+    return;
 }
 
 
@@ -1124,6 +1157,7 @@ sub tsetAddMember
 
     $self->{'db_sets'}->addToList('s:'.$tset, $token);
     $self->{'db_sets'}->put('o:'.$tset.':'.$token, $origin);
+    return;
 }
 
 
@@ -1135,6 +1169,7 @@ sub tsetDelMember
 
     $self->{'db_sets'}->delFromList('s:'.$tset, $token);
     $self->{'db_sets'}->del('o:'.$tset.':'.$token);
+    return;
 }
 
 # Definitions manipulation
