@@ -14,17 +14,18 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 
 package Torrus::ConfigTree::Validator;
 
+use strict;
+use warnings;
+
 use Torrus::ConfigTree;
 use Torrus::Log;
 use Torrus::RPN;
 use Torrus::SiteConfig;
-use strict;
 
 Torrus::SiteConfig::loadStyling();
 
@@ -138,10 +139,16 @@ my %view_params =
 
 foreach my $mod ( @Torrus::Validator::loadLeafValidators )
 {
-    eval( 'require ' . $mod );
-    die( $@ ) if $@;
-    eval( '&' . $mod . '::initValidatorLeafParams( \%leaf_params )' );
-    die( $@ ) if $@;
+    if( not eval('require ' . $mod) or $@ )
+    {
+        die($@);
+    }
+    
+    if( not eval('&' . $mod . '::initValidatorLeafParams(\%leaf_params); 1;')
+        or $@ )
+    {
+        die($@);
+    }
 }
 
 
@@ -352,17 +359,20 @@ sub validateNode
             }
             foreach my $dname ( @dsNames )
             {
-                my $param = 'ds-expr-' . $dname;
-                my $expr = $config_tree->getNodeParam( $token, $param );
-                if( not defined( $expr ) )
                 {
-                    my $path = $config_tree->path( $token );
-                    Error("Parameter $param is not defined in $path");
-                    $ok = 0;
-                }
-                else
-                {
-                    $ok = validateRPN( $token, $expr, $config_tree ) ? $ok : 0;
+                    my $param = 'ds-expr-' . $dname;
+                    my $expr = $config_tree->getNodeParam( $token, $param );
+                    if( not defined( $expr ) )
+                    {
+                        my $path = $config_tree->path( $token );
+                        Error("Parameter $param is not defined in $path");
+                        $ok = 0;
+                    }
+                    else
+                    {
+                        $ok = validateRPN( $token, $expr, $config_tree ) ?
+                            $ok : 0;
+                    }
                 }
 
                 foreach my $paramprefix ( 'graph-legend-', 'line-style-',
