@@ -14,7 +14,6 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 #
@@ -23,15 +22,16 @@
 
 package Torrus::ConfigTree::Writer;
 
-use Torrus::ConfigTree;
-our @ISA=qw(Torrus::ConfigTree);
+use strict;
+use warnings;
+
+use base 'Torrus::ConfigTree';
 
 use Torrus::Log;
 use Torrus::TimeStamp;
 use Torrus::SiteConfig;
 use Torrus::ServiceID;
     
-use strict;
 use Digest::MD5 qw(md5); # needed as hash function
 
 
@@ -111,6 +111,7 @@ sub setParam
     $self->{'paramcache'}{$name}{$param} = $value;
     $self->{'db_otherconfig'}->put( 'P:'.$name.':'.$param, $value );
     $self->{'db_otherconfig'}->addToList('Pl:'.$name, $param);
+    return;
 }
 
 sub setNodeParam
@@ -128,6 +129,7 @@ sub setNodeParam
     $self->{'paramcache'}{$name}{$param} = $value;
     $self->{'db_dsconfig'}->put( 'P:'.$name.':'.$param, $value );
     $self->{'db_dsconfig'}->addToList('Pl:'.$name, $param);
+    return;
 }
 
 
@@ -140,6 +142,7 @@ sub setParamProperty
 
     $self->{'paramprop'}{$prop}{$param} = $value;
     $self->{'db_paramprops'}->put( $param . ':' . $prop, $value );
+    return;
 }
 
 
@@ -154,6 +157,7 @@ sub initRoot
         $self->{'db_dsconfig'}->put( 'n:'.$token, 0 );
         $self->{'nodetype_cache'}{$token} = 0;
     }
+    return;
 }
 
 sub addChild
@@ -276,6 +280,7 @@ sub addView
     {
         $self->{'viewparent'}{$vname} = $parent;
     }
+    return;
 }
 
 
@@ -284,6 +289,7 @@ sub addMonitor
     my $self = shift;
     my $mname = shift;
     $self->{'db_otherconfig'}->addToList('M:', $mname);
+    return;
 }
 
 
@@ -292,6 +298,7 @@ sub addAction
     my $self = shift;
     my $aname = shift;
     $self->{'db_otherconfig'}->addToList('A:', $aname);
+    return;
 }
 
 
@@ -302,6 +309,7 @@ sub addDefinition
     my $value = shift;
     $self->{'db_dsconfig'}->put( 'd:'.$name, $value );
     $self->{'db_dsconfig'}->addToList('D:', $name);
+    return;
 }
 
 
@@ -313,6 +321,7 @@ sub setVar
     my $value = shift;
     
     $self->{'setvar'}{$token}{$name} = $value;
+    return;
 }
 
 
@@ -378,6 +387,7 @@ sub finalize
         Torrus::TimeStamp::setNow($self->treeName() . ':configuration');
         Torrus::TimeStamp::release();
     }
+    return;
 }
 
 
@@ -723,6 +733,7 @@ sub propagateViewParams
 
     # mark this view as processed
     $self->{'viewParamsProcessed'}{$vname} = 1;
+    return;
 }
 
 
