@@ -14,14 +14,15 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 
 package Torrus::ConfigTree::XMLCompiler;
 
-use Torrus::ConfigTree::Writer;
-our @ISA=qw(Torrus::ConfigTree::Writer);
+use strict;
+use warnings;
+
+use base 'Torrus::ConfigTree::Writer';
 
 use Torrus::ConfigTree;
 use Torrus::ConfigTree::Validator;
@@ -30,7 +31,6 @@ use Torrus::Log;
 use Torrus::TimeStamp;
 
 use XML::LibXML;
-use strict;
 
 sub new
 {
@@ -87,8 +87,7 @@ sub compile
     my $ok = 1;
     my $parser = new XML::LibXML;
     my $doc;
-    eval { $doc = $parser->parse_file( $filename );  };
-    if( $@ )
+    if( not eval {$doc = $parser->parse_file($filename)} or $@ )
     {
         Error("Failed to parse $filename: $@");
         return 0;
@@ -99,10 +98,8 @@ sub compile
     # Initialize the '/' element
     $self->initRoot();
 
-    my $node;
-
     # First of all process all pre-required files
-    foreach $node ( $root->getElementsByTagName('include') )
+    foreach my $node ( $root->getElementsByTagName('include') )
     {
         my $incfile = $node->getAttribute('filename');
         if( not $incfile )
@@ -116,35 +113,35 @@ sub compile
         }
     }
 
-    foreach $node ( $root->getElementsByTagName('param-properties') )
+    foreach my $node ( $root->getElementsByTagName('param-properties') )
     {
         $ok = $self->compile_paramprops( $node ) ? $ok:0;
     }
 
     if( not $self->{'-NoDSRebuild'} )
     {
-        foreach $node ( $root->getElementsByTagName('definitions') )
+        foreach my $node ( $root->getElementsByTagName('definitions') )
         {
             $ok = $self->compile_definitions( $node ) ? $ok:0;
         }
 
-        foreach $node ( $root->getElementsByTagName('datasources') )
+        foreach my $node ( $root->getElementsByTagName('datasources') )
         {
             $ok = $self->compile_ds( $node ) ? $ok:0;
         }
     }
 
-    foreach $node ( $root->getElementsByTagName('monitors') )
+    foreach my $node ( $root->getElementsByTagName('monitors') )
     {
         $ok = $self->compile_monitors( $node ) ? $ok:0;
     }
 
-    foreach $node ( $root->getElementsByTagName('token-sets') )
+    foreach my $node ( $root->getElementsByTagName('token-sets') )
     {
         $ok = $self->compile_tokensets( $node ) ? $ok:0;
     }
 
-    foreach $node ( $root->getElementsByTagName('views') )
+    foreach my $node ( $root->getElementsByTagName('views') )
     {
         $ok = $self->compile_views( $node ) ? $ok:0;
     }
@@ -311,6 +308,23 @@ sub compile_subtrees
     
     my $ok = 1;
 
+    # setting of compile-time variables
+    
+    foreach my $setvar ( $node->getChildrenByTagName('setvar') )        
+    {
+        my $name = $setvar->getAttribute('name');
+        my $value = $setvar->getAttribute('value');
+        if( not defined( $name ) or not defined( $value ) )
+        {
+            my $path = $self->path($token);
+            Error("Setvar statement without name or value in $path"); $ok = 0;
+        }
+        else
+        {
+            $self->setVar( $token, $name, $value );
+        }
+    }
+
     # Apply templates
 
     foreach my $templateapp ( $node->getChildrenByTagName('apply-template') )
@@ -348,22 +362,8 @@ sub compile_subtrees
         $ok = $self->setAlias($token, $apath) ? $ok:0;
     }
 
-    foreach my $setvar ( $node->getChildrenByTagName('setvar') )        
-    {
-        my $name = $setvar->getAttribute('name');
-        my $value = $setvar->getAttribute('value');
-        if( not defined( $name ) or not defined( $value ) )
-        {
-            my $path = $self->path($token);
-            Error("Setvar statement without name or value in $path"); $ok = 0;
-        }
-        else
-        {
-            $self->setVar( $token, $name, $value );
-        }
-    }
 
-    # Compile-time variables
+    # applying compile-time variables
     
     foreach my $iftrue ( $node->getChildrenByTagName('iftrue') )        
     {
@@ -464,7 +464,7 @@ sub compile_monitors
         }
         else
         {
-            $ok = $self->addMonitor( $mname );
+            $self->addMonitor( $mname );
             $ok = $self->compile_params($monitor, $mname) ? $ok:0;
         }
     }
@@ -504,7 +504,7 @@ sub compile_tokensets
         else
         {
             $sname = 'S'. $sname;
-            $ok = $self->addTset( $sname );
+            $self->addTset( $sname );
             $ok = $self->compile_params($tokenset, $sname) ? $ok:0;
         }
     }
