@@ -538,24 +538,23 @@ sub buildConfig
             else
             {
                 push( @{$data->{'templates'}}, '::snmp-defaults' );
+
+                if( $devdetails->paramDisabled('disable-reachability-stats') 
+                    and
+                    (
+                     (not defined($devdetails->param('only-devtypes')))
+                     or
+                     $devdetails->paramEnabled('enable-reachability-stats') 
+                    ) )
+                {
+                    push( @{$data->{'templates'}}, '::snmp-reachability' );
+                }
             }
 
             if( $devdetails->paramEnabled('rrd-hwpredict' ) )
             {
                 push( @{$data->{'templates'}}, '::holt-winters-defaults' );
-            }
-            
-            if( $devdetails->paramDisabled('disable-reachability-stats') 
-                and
-                (
-                 (not defined($devdetails->param('only-devtypes')))
-                 or
-                 $devdetails->paramEnabled('enable-reachability-stats') 
-                ) )
-            {
-                push( @{$data->{'templates'}}, '::snmp-reachability' );
-            }
-
+            }            
             
             my $devNodeName = $devdetails->paramString('symbolic-name');
             if( $devNodeName eq '' )
@@ -594,7 +593,7 @@ sub buildConfig
             {
                 &{$reg->{$devtype}{'buildConfig'}}
                 ( $devdetails, $cb, $devNode, $self->{'globalData'} );
-            }
+          }
 
             $cb->{'statistics'}{'hosts'}++;
         }
@@ -732,10 +731,15 @@ sub walkSnmpTable
 
     my $session = $self->session();
     my $base = $self->oiddef( $oidname );
+
+    my @arg = ( -baseoid => $base );
+
+    if( $session->version() > 0 )
+    {
+        push(@arg, -maxrepetitions => $self->{'maxrepetitions'});
+    }
     
-    my $table = $session->get_table
-        ( -baseoid => $base,
-          -maxrepetitions => $self->{'maxrepetitions'} );
+    my $table = $session->get_table( @arg );
     
     if( not defined($table) )
     {
