@@ -15,7 +15,6 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-# $Id$
 # Marc Haber <mh+torrus-devel@zugschlus.de>
 # Redesigned by Stanislav Sinyagin
 
@@ -24,6 +23,8 @@
 package Torrus::DevDiscover::AlliedTelesyn_PBC18;
 
 use strict;
+use warnings;
+
 use Torrus::Log;
 
 
@@ -85,9 +86,9 @@ sub discover
     
     # Modules table
 
-    my $oid = $dd->oiddef('ATMCCommon-MIB::mcModuleType');
+    my $base = $dd->oiddef('ATMCCommon-MIB::mcModuleType');
     
-    my $table = $session->get_table( -baseoid => $oid );    
+    my $table = $session->get_table( -baseoid => $base );    
     if( not defined( $table ) )
     {
         return 0;
@@ -95,9 +96,9 @@ sub discover
     
     $devdetails->storeSnmpVars( $table );
     
-    foreach my $INDEX ( $devdetails->getSnmpIndices($oid) )
+    foreach my $INDEX ( $devdetails->getSnmpIndices($base) )
     {
-        my $moduleType = $devdetails->snmpVar( $oid . '.' . $INDEX );
+        my $moduleType = $devdetails->snmpVar( $base . '.' . $INDEX );
         if( $moduleType == 0 )
         {
             next;
@@ -153,7 +154,7 @@ sub discover
         my $name = $devdetails->snmpVar
             ( $dd->oiddef('ATMCCommon-MIB::mcModuleName') .'.'.$INDEX );
 
-        if( length( $name ) > 0 )
+        if( defined($name) and $name ne '' )
         {
             $data->{'PBC18'}{$INDEX}{'moduleName'} = $name;
         }
@@ -215,21 +216,22 @@ sub buildConfig
         
     foreach my $INDEX ( sort {$a<=>$b} keys %{$data->{'PBC18'}} )
     {
-        my $param = { 'pbc-module-index' => $INDEX };
+        my $modParam = { 'pbc-module-index' => $INDEX };
         
         if( defined( $data->{'PBC18'}{$INDEX}{'moduleDesc'} ) )
         {
-            $param->{'legend'} =
+            $modParam->{'legend'} =
                 'Module type: ' . $data->{'PBC18'}{$INDEX}{'moduleDesc'};
         }
 
         if( defined( $data->{'PBC18'}{$INDEX}{'moduleName'} ) )
         {
-            $param->{'comment'} =
+            $modParam->{'comment'} =
                 $data->{'PBC18'}{$INDEX}{'moduleName'};
         }
         
-        my $modNode = $cb->addSubtree( $devNode, 'Module_' . $INDEX, $param );
+        my $modNode =
+            $cb->addSubtree( $devNode, 'Module_' . $INDEX, $modParam );
 
         my $mgParam = {
             'ds-type'              => 'rrd-multigraph',
@@ -258,18 +260,19 @@ sub buildConfig
             $mgParam->{'line-order-' . $dsname} = $n;
             $mgParam->{'ds-expr-' . $dsname} = '{Port_' . $portName . '}';
             
-            my $param = {
+            my $portParam = {
                 'rrd-ds' => 'Port' . $portName,
                 'snmp-object' =>
                     $data->{'PBC18'}{$INDEX}{'portAvailable'}{$portName},
                 };
 
-            $cb->addLeaf( $modNode, 'Port_' . $portName, $param );
+            $cb->addLeaf( $modNode, 'Port_' . $portName, $portParam );
             $n++;
         }
 
         $cb->addLeaf( $modNode, 'AllPorts', $mgParam );        
     }
+    return;
 }
 
 
