@@ -14,7 +14,6 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Cisco CatOS devices discovery
@@ -25,6 +24,8 @@
 package Torrus::DevDiscover::CiscoCatOS;
 
 use strict;
+use warnings;
+
 use Torrus::Log;
 
 
@@ -142,14 +143,14 @@ sub discover
     }
 
     # In large installations, only named ports may be of interest
-    if( $devdetails->param('CiscoCatOS::suppress-noname-ports') eq 'yes' )
+    if( $devdetails->paramEnabled('CiscoCatOS::suppress-noname-ports') )
     {
         my $nExcluded = 0;
         foreach my $ifIndex ( keys %{$data->{'interfaces'}} )
         {
             my $interface = $data->{'interfaces'}{$ifIndex};
-            if( not defined( $interface->{'portName'} ) or
-                length( $interface->{'portName'} ) == 0 )
+            if( not defined($interface->{'portName'}) or
+                $interface->{'portName'} eq '' )
             {
                 $interface->{'excluded'} = 1;
                 $nExcluded++;
@@ -180,6 +181,7 @@ sub buildConfig
     my $devdetails = shift;
     my $cb = shift;
     my $devNode = shift;
+    return;
 }
 
 
