@@ -14,7 +14,6 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Common Cisco MIBs, supported by many IOS and CatOS devices
@@ -22,6 +21,8 @@
 package Torrus::DevDiscover::CiscoGeneric;
 
 use strict;
+use warnings;
+
 use Torrus::Log;
 
 
@@ -84,7 +85,7 @@ sub discover
     my $session = $dd->session();
     my $data = $devdetails->data();
 
-    if( $devdetails->param('CiscoGeneric::disable-sensors') ne 'yes' )
+    if( $devdetails->paramDisabled('CiscoGeneric::disable-sensors') )
     {
         # Check if temperature sensors are supported
 
@@ -122,7 +123,7 @@ sub discover
         }
     }
 
-    if( $devdetails->param('CiscoGeneric::disable-psupplies') ne 'yes' )
+    if( $devdetails->paramDisabled('CiscoGeneric::disable-psupplies') )
     {
         # Check if power supply status is supported
 
@@ -149,12 +150,13 @@ sub discover
         }
     }
     
-    if( $devdetails->param('CiscoGeneric::disable-memory-pools') ne 'yes' )
+    if( $devdetails->paramDisabled('CiscoGeneric::disable-memory-pools') )
     {
         my $eMemPool =
             $session->get_table( -baseoid =>
                                  $dd->oiddef('cempMemPoolName') );
-        if( defined $eMemPool and scalar( %{$eMemPool} ) > 0 and
+        
+        if( defined($eMemPool) and scalar(keys %{$eMemPool}) > 0 and
             $devdetails->isDevType('RFC2737_ENTITY_MIB') )
         {
             $devdetails->storeSnmpVars( $eMemPool );
@@ -195,7 +197,7 @@ sub discover
                 $session->get_table( -baseoid =>
                                      $dd->oiddef('ciscoMemoryPoolName') );
 
-            if( defined $MemoryPool and scalar( %{$MemoryPool} ) > 0 )
+            if( defined($MemoryPool) and scalar(keys %{$MemoryPool}) > 0 )
             {
                 $devdetails->storeSnmpVars( $MemoryPool );
                 $devdetails->setCap('ciscoMemoryPool');
@@ -223,7 +225,7 @@ sub discover
         }
     }
 
-    if( $devdetails->param('CiscoGeneric::disable-cpu-stats') ne 'yes' )
+    if( $devdetails->paramDisabled('CiscoGeneric::disable-cpu-stats') )
     {
         my $ciscoCpuStats =
             $session->get_table( -baseoid => $dd->oiddef('cpmCPUTotalTable') );
@@ -328,22 +330,23 @@ sub buildConfig
         my $subtreeName = 'Temperature_Sensors';
 
         my $fahrenheit =
-            $devdetails->param('CiscoGeneric::use-fahrenheit') eq 'yes';
+            $devdetails->paramEnabled('CiscoGeneric::use-fahrenheit');
 
-        my $param = {
+        my $subtreeParam = {
             'node-display-name' => 'Temperature Sensors',
         };
-        my $templates = [ 'CiscoGeneric::cisco-temperature-subtree' ];
         
         my $filePerSensor =
-            $devdetails->param('CiscoGeneric::file-per-sensor') eq 'yes';
+            $devdetails->paramEnabled('CiscoGeneric::file-per-sensor');
         
-        $param->{'data-file'} = '%snmp-host%_sensors' .
+        $subtreeParam->{'data-file'} = '%snmp-host%_sensors' .
             ($filePerSensor ? '_%sensor-index%':'') .
             ($fahrenheit ? '_fahrenheit':'') . '.rrd';
 
-        my $subtreeNode = $cb->addSubtree( $devNode, $subtreeName,
-                                           $param, $templates );
+        my $subtreeNode =
+            $cb->addSubtree( $devNode, $subtreeName,
+                             $subtreeParam,
+                             [ 'CiscoGeneric::cisco-temperature-subtree' ] );
         
         foreach my $sIndex ( sort {$a<=>$b} keys
                              %{$data->{'ciscoTemperatureSensors'}} )
@@ -366,8 +369,9 @@ sub buildConfig
                 'upper-limit'        => $threshold
                 };
 
-            my $templates = ['CiscoGeneric::cisco-temperature-sensor' .
-                             ($fahrenheit ? '-fahrenheit':'')];
+            my $templates =
+                ['CiscoGeneric::cisco-temperature-sensor' .
+                 ($fahrenheit ? '-fahrenheit':'')];
 
             my $monitor = $data->{'ciscoTemperatureSensors'}{$sIndex}->{
                 'selectorActions'}{'Monitor'};
@@ -394,23 +398,22 @@ sub buildConfig
         # Create a subtree for the power supplies
         my $subtreeName = 'Power_Supplies';
 
-        my $param = {
+        my $subtreeParam = {
             'node-display-name' => 'Power Supplies',
             'comment' => 'Power supplies status',
             'precedence' => -600,
         };
-        my $templates = [];
                 
-        $param->{'data-file'} = '%system-id%_power.rrd';
+        $subtreeParam->{'data-file'} = '%system-id%_power.rrd';
 
-        my $monitor = $devdetails->param('CiscoGeneric::power-monitor');
-        if( length( $monitor ) > 0 )
+        my $monitor = $devdetails->paramString('CiscoGeneric::power-monitor');
+        if( $monitor ne '' )
         {
-            $param->{'monitor'} = $monitor;
+            $subtreeParam->{'monitor'} = $monitor;
         }
 
-        my $subtreeNode = $cb->addSubtree( $devNode, $subtreeName,
-                                           $param, $templates );
+        my $subtreeNode =
+            $cb->addSubtree( $devNode, $subtreeName, $subtreeParam );
         
         foreach my $sIndex ( sort {$a<=>$b} @{$data->{'ciscoPowerSupplies'}} )
         {
@@ -420,9 +423,8 @@ sub buildConfig
                 'power-index'       => $sIndex
                 };
 
-            my $templates = ['CiscoGeneric::cisco-power-supply'];
-
-            $cb->addLeaf( $subtreeNode, $leafName, $param, $templates );
+            $cb->addLeaf( $subtreeNode, $leafName, $param,
+                          ['CiscoGeneric::cisco-power-supply'] );
         }
     }
 
@@ -434,14 +436,14 @@ sub buildConfig
     {
         my $subtreeName = 'Memory_Usage';
 
-        my $param = {
+        my $subtreeParam = {
             'node-display-name' => 'Memory Usage',
             'precedence'        => '-100',
             'comment'           => 'Router memory utilization'
             };
 
         my $subtreeNode =
-            $cb->addSubtree( $devNode, $subtreeName, $param,
+            $cb->addSubtree( $devNode, $subtreeName, $subtreeParam,
                              ['CiscoGeneric::cisco-memusage-subtree']);
 
         if( $devdetails->hasCap('cempMemPool') )
@@ -524,14 +526,14 @@ sub buildConfig
     if( $devdetails->hasCap('ciscoCpuStats') )
     {
         my $subtreeName = 'CPU_Usage';
-        my $param = {
+        my $subtreeParam = {
             'node-display-name' => 'CPU Usage',
             'precedence'         => '-500',
             'comment'            => 'Overall CPU busy percentage'
             };
 
         my $subtreeNode =
-            $cb->addSubtree( $devNode, $subtreeName, $param,
+            $cb->addSubtree( $devNode, $subtreeName, $subtreeParam,
                              ['CiscoGeneric::cisco-cpu-usage-subtree']);
         
         foreach my $INDEX ( sort {$a<=>$b} keys %{$data->{'ciscoCpuStats'}} )
@@ -558,7 +560,8 @@ sub buildConfig
             
             my @templates;
 
-            if( $cpu->{'stats-type'} eq 'revised' )
+            if( defined($cpu->{'stats-type'}) and
+                $cpu->{'stats-type'} eq 'revised' )
             {
                 push( @templates, 'CiscoGeneric::cisco-cpu-revised' );
             }
@@ -578,6 +581,8 @@ sub buildConfig
             }
         }
     }
+
+    return;
 }
 
 
@@ -600,7 +605,6 @@ $Torrus::DevDiscover::selectorsRegistry{'CiscoCPU'} = {
     'applyAction'     => \&applySelectorAction,
 };
 
-## Objects are interface indexes
 
 sub getSelectorObjects
 {
@@ -729,6 +733,8 @@ sub applySelectorAction
     {
         Error('Unknown Cisco selector action: ' . $action);
     }
+
+    return;
 }   
 
 
