@@ -14,17 +14,15 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Cisco IOS devices discovery
-# To do:
-#   SA Agent MIB
-#   DiffServ MIB
 
 package Torrus::DevDiscover::CiscoIOS;
 
 use strict;
+use warnings;
+
 use Torrus::Log;
 
 
@@ -55,7 +53,6 @@ our %oiddef =
      'cbgpPeerAcceptedPrefixes'          => '1.3.6.1.4.1.9.9.187.1.2.4.1.1',
      'cbgpPeerPrefixAdminLimit'          => '1.3.6.1.4.1.9.9.187.1.2.4.1.3',
      # CISCO-CAR-MIB
-     'ccarConfigTable'                   => '1.3.6.1.4.1.9.9.113.1.1.1',
      'ccarConfigType'                    => '1.3.6.1.4.1.9.9.113.1.1.1.1.3',
      'ccarConfigAccIdx'                  => '1.3.6.1.4.1.9.9.113.1.1.1.1.4',
      'ccarConfigRate'                    => '1.3.6.1.4.1.9.9.113.1.1.1.1.5',
@@ -189,7 +186,7 @@ sub checkdevtype
     # On some Layer3 switching devices, VlanXXX interfaces give some
     # useful stats, while on others the stats are not relevant at all
     
-    if( $devdetails->param('CiscoIOS::enable-vlan-interfaces') ne 'yes' )
+    if( $devdetails->paramDisabled('CiscoIOS::enable-vlan-interfaces') )
     {
         $interfaceFilter->{'VlanN'} = {
             'ifType'  => 53,                     # propVirtual
@@ -198,8 +195,8 @@ sub checkdevtype
     }
 
     # same thing with unrouted VLAN interfaces
-    if( $devdetails->param('CiscoIOS::enable-unrouted-vlan-interfaces')
-        ne 'yes' )
+    if( $devdetails->paramDisabled
+        ('CiscoIOS::enable-unrouted-vlan-interfaces'))
     {
         $interfaceFilter->{'unrouted VLAN N'} = {
             'ifType'  => 53,                     # propVirtual
@@ -208,13 +205,16 @@ sub checkdevtype
     }
 
     # sometimes we want Dialer interface stats
-    if( $devdetails->param('CiscoIOS::enable-dialer-interfaces')
-        ne 'yes' )
+    if( $devdetails->paramDisabled('CiscoIOS::enable-dialer-interfaces') )
     {
         $interfaceFilter->{'DialerN'} = {
             'ifType'  => 23,                     # ppp
             'ifDescr' => '^Dialer'
-        },
+            };
+        $interfaceFilter->{'DialerN_ML'} = {
+            'ifType'  => 108,                    # pppMultilinkBundle
+            'ifDescr' => '^Dialer'
+            };
     }
 
     &Torrus::DevDiscover::RFC2863_IF_MIB::addInterfaceFilter
@@ -232,12 +232,12 @@ sub checkdevtype
 }
 
 
-my %ccarConfigType =
+my %ccarConfigTypeTranslation =
     ( 1 => 'all',
       2 => 'quickAcc',
       3 => 'standardAcc' );
 
-my %ccarAction =
+my %ccarActionTranslation =
     ( 1 => 'drop',
       2 => 'xmit',
       3 => 'continue',
@@ -254,6 +254,12 @@ sub discover
     my $session = $dd->session();
     my $data = $devdetails->data();
 
+    # fixing the interface indexing and referencing schema
+    $data->{'nameref'}{'ifSubtreeName'}   = 'ifDescrT';
+    $data->{'nameref'}{'ifReferenceName'} = 'ifDescr';
+    $data->{'nameref'}{'ifNick'} = 'ifNameT';
+    $data->{'nameref'}{'ifComment'} = 'ifAlias';
+
     # Old mkroutercfg used cisco-interface-counters
     if( $Torrus::DevDiscover::CiscoIOS::useCiscoInterfaceCounters )
     {
@@ -265,40 +271,8 @@ sub discover
                   'CiscoIOS::cisco-interface-counters' );
         }
     }
-    else
-    {
-        # This is a well-known bug in IOS: HC counters are implemented,
-        # but always zero. We can catch this only for active interfaces.
 
-        foreach my $ifIndex ( sort {$a<=>$b} keys %{$data->{'interfaces'}} )
-        {
-            my $interface = $data->{'interfaces'}{$ifIndex};
-
-            if( $interface->{'hasHCOctets'} and
-                ( (
-                   $devdetails->snmpVar( $dd->oiddef('ifHCInOctets')
-                                         . '.' . $ifIndex ) == 0 and
-                   $devdetails->snmpVar( $dd->oiddef('ifInOctets')
-                                         . '.' . $ifIndex ) > 0
-                  )
-                  or
-                  (
-                   $devdetails->snmpVar( $dd->oiddef('ifHCOutOctets')
-                                         . '.' . $ifIndex ) == 0 and
-                   $devdetails->snmpVar( $dd->oiddef('ifOutOctets')
-                                         . '.' . $ifIndex ) > 0
-                  ) ) )
-            {
-                Debug('Disabling HC octets for ' . $ifIndex . ': ' .
-                      $interface->{'ifDescr'});
-
-                $interface->{'hasHCOctets'} = 0;
-                $interface->{'hasHCUcastPkts'} = 0;
-            }
-        }
-    }
-
-    if( $devdetails->param('CiscoIOS::enable-membuf-stats') eq 'yes' )
+    if( $devdetails->paramEnabled('CiscoIOS::enable-membuf-stats') )
     {
         # Old Memory Buffers, if we have bufferElFree we assume
         # the rest as they are "required"
@@ -311,7 +285,7 @@ sub discover
         }
     }
 
-    if( $devdetails->param('CiscoIOS::disable-ipsec-stats') ne 'yes' )
+    if( $devdetails->paramDisabled('CiscoIOS::disable-ipsec-stats') )
     {
         if( $dd->checkSnmpOID('cipSecGlobalHcInOctets') )
         {
@@ -328,20 +302,14 @@ sub discover
         }
     }
 
-    if( $devdetails->param('CiscoIOS::disable-bgp-stats') ne 'yes' )
+    if( $devdetails->paramDisabled('CiscoIOS::disable-bgp-stats') )
     {
-        my $peerTable =
-            $session->get_table( -baseoid =>
-                                 $dd->oiddef('cbgpPeerAcceptedPrefixes') );
-        if( defined( $peerTable ) and scalar( %{$peerTable} ) > 0 )
+        my $peerTable = $dd->walkSnmpTable('cbgpPeerAcceptedPrefixes');
+        if( scalar(keys %{$peerTable}) > 0 )
         {
-            $devdetails->storeSnmpVars( $peerTable );
             $devdetails->setCap('CiscoBGP');
 
-            my $limitsTable =
-                $session->get_table( -baseoid =>
-                                     $dd->oiddef('cbgpPeerPrefixAdminLimit') );
-            $limitsTable = {} if not defined( $limitsTable );
+            my $limitsTable = $dd->walkSnmpTable('cbgpPeerPrefixAdminLimit');
             
             $data->{'cbgpPeers'} = {};
             
@@ -354,9 +322,7 @@ sub discover
             # Number of peers for each AS
             my %asNumbers;    
 
-            foreach my $INDEX
-                ( $devdetails->
-                  getSnmpIndices( $dd->oiddef('cbgpPeerAcceptedPrefixes') ) )
+            foreach my $INDEX ( keys %{$peerTable} )
             {
                 my ($a1, $a2, $a3, $a4, $afi, $safi) = split(/\./, $INDEX);
                 my $peerIP = join('.', $a1, $a2, $a3, $a4);
@@ -371,13 +337,16 @@ sub discover
                     push( @nonV4Unicast, $INDEX );
                 }
 
-                my $desc =
-                    $devdetails->param('peer-ipaddr-description-' .
-                                       join('_', split('\.', $peerIP)));
-                if( length( $desc ) > 0 )
                 {
-                    $peer->{'description'} = $desc;
-                }        
+                    my $desc =
+                        $devdetails->paramString
+                        ('peer-ipaddr-description-' .
+                         join('_', split('\.', $peerIP)));
+                    if( $desc ne '' )
+                    {
+                        $peer->{'description'} = $desc;
+                    }
+                }
                 
                 my $peerAS = $data->{'bgpPeerAS'}{$peerIP};
                 if( defined( $peerAS ) )
@@ -386,8 +355,9 @@ sub discover
                     $asNumbers{$peer->{'peerAS'}}++;
 
                     my $desc =
-                        $devdetails->param('bgp-as-description-' . $peerAS);
-                    if( length( $desc ) > 0 )
+                        $devdetails->paramString
+                        ('bgp-as-description-' . $peerAS);
+                    if( $desc ne '' )
                     {
                         if( defined( $peer->{'description'} ) )
                         {
@@ -409,9 +379,7 @@ sub discover
                 }
                 $peer->{'description'} .= '[' . $peerIP . ']';
 
-                $peer->{'prefixLimit'} =
-                    $limitsTable->{$dd->oiddef('cbgpPeerPrefixAdminLimit') .
-                                       '.' . $INDEX};
+                $peer->{'prefixLimit'} =  $limitsTable->{$INDEX};
                 
                 $data->{'cbgpPeers'}{$INDEX} = $peer;                
             }
@@ -419,17 +387,12 @@ sub discover
             if( scalar( @nonV4Unicast ) > 0 )
             {
                 my $addrFamTable =
-                    $session->get_table
-                    ( -baseoid => $dd->oiddef('cbgpPeerAddrFamilyName') );
+                    $dd->walkSnmpTable('cbgpPeerAddrFamilyName');
                 
                 foreach my $INDEX ( @nonV4Unicast )
                 {
                     my $peer = $data->{'cbgpPeers'}{$INDEX};
-
-                    my $fam = $addrFamTable->{
-                        $dd->oiddef('cbgpPeerAddrFamilyName') .
-                            '.' . $INDEX};
-
+                    my $fam = $addrFamTable->{$INDEX};
                     $peer->{'addrFamily'} = $fam;
                     $peer->{'otherAddrFamily'} = 1;
                     $peer->{'description'} .= ' ' . $fam;
@@ -461,24 +424,31 @@ sub discover
     }
 
     
-    if( $devdetails->param('CiscoIOS::disable-car-stats') ne 'yes' )
+    if( $devdetails->paramDisabled('CiscoIOS::disable-car-stats') )
     {
-        my $carTable =
-            $session->get_table( -baseoid =>
-                                 $dd->oiddef('ccarConfigTable') );
-        if( defined( $carTable ) and scalar( %{$carTable} ) > 0 )
+        my $ccarConfigType = $dd->walkSnmpTable('ccarConfigType');
+        if( scalar(keys %{$ccarConfigType}) > 0 )
         {
-            $devdetails->storeSnmpVars( $carTable );
             $devdetails->setCap('CiscoCAR');
-
             $data->{'ccar'} = {};
 
-            foreach my $INDEX
-                ( $devdetails->
-                  getSnmpIndices( $dd->oiddef('ccarConfigType') ) )
+            my $ccarConfigAccIdx = $dd->walkSnmpTable('ccarConfigAccIdx');
+            my $ccarConfigRate = $dd->walkSnmpTable('ccarConfigRate');
+            my $ccarConfigLimit = $dd->walkSnmpTable('ccarConfigLimit');
+            my $ccarConfigExtLimit = $dd->walkSnmpTable('ccarConfigExtLimit');
+            my $ccarConfigConformAction =
+                $dd->walkSnmpTable('ccarConfigConformAction');
+            my $ccarConfigExceedAction =
+                $dd->walkSnmpTable('ccarConfigExceedAction');
+            
+            while( my($INDEX, $confType) = each %{$ccarConfigType} )
             {
                 my ($ifIndex, $dir, $carIndex) = split(/\./, $INDEX);
                 my $interface = $data->{'interfaces'}{$ifIndex};
+                if( not defined($interface) or $interface->{'excluded'} )
+                {
+                    next;
+                }
 
                 my $car = {
                     'ifIndex'   => $ifIndex,
@@ -486,35 +456,17 @@ sub discover
                     'carIndex'  => $carIndex };
 
                 $car->{'configType'} =
-                    $ccarConfigType{ $carTable->{$dd->oiddef
-                                                     ('ccarConfigType') .
-                                                     '.' . $INDEX} };
-
-                $car->{'accIdx'} = $carTable->{$dd->oiddef
-                                                   ('ccarConfigAccIdx') .
-                                                   '.' . $INDEX};
-                
-                $car->{'rate'} = $carTable->{$dd->oiddef
-                                                 ('ccarConfigRate') .
-                                                 '.' . $INDEX};
-
-                
-                $car->{'limit'} = $carTable->{$dd->oiddef
-                                                  ('ccarConfigLimit') .
-                                                  '.' . $INDEX};
-                
-                $car->{'extLimit'} = $carTable->{$dd->oiddef
-                                                     ('ccarConfigExtLimit') .
-                                                     '.' . $INDEX};
+                    $ccarConfigTypeTranslation{ $ccarConfigType->{$INDEX} };
+                $car->{'accIdx'} = $ccarConfigAccIdx->{$INDEX};
+                $car->{'rate'} = $ccarConfigRate->{$INDEX};
+                $car->{'limit'} = $ccarConfigLimit->{$INDEX};
+                $car->{'extLimit'} = $ccarConfigExtLimit->{$INDEX};
                 $car->{'conformAction'} =
-                    $ccarAction{ $carTable->{$dd->oiddef
-                                                 ('ccarConfigConformAction') .
-                                                 '.' . $INDEX} };
-                
+                    $ccarActionTranslation{
+                        $ccarConfigConformAction->{$INDEX} };
                 $car->{'exceedAction'} =
-                    $ccarAction{ $carTable->{$dd->oiddef
-                                                 ('ccarConfigExceedAction') .
-                                                 '.' . $INDEX} };
+                    $ccarActionTranslation{
+                        $ccarConfigExceedAction->{$INDEX} };
 
                 $data->{'ccar'}{$INDEX} = $car;
             }
@@ -522,89 +474,75 @@ sub discover
     }
 
 
-    if( $devdetails->param('CiscoIOS::disable-vpdn-stats') ne 'yes' )
+    if( $devdetails->paramDisabled('CiscoIOS::disable-vpdn-stats') )
     {
         if( $dd->checkSnmpTable( 'cvpdnSystemTunnelTotal' ) )
         {
             # Find the Tunnel type
-            my $tableTun = $session->get_table(
-                -baseoid => $dd->oiddef('cvpdnSystemTunnelTotal') );
-
-            if( $tableTun )
+            my $cvpdnSystemTunnelTotal =
+                $dd->walkSnmpTable('cvpdnSystemTunnelTotal');
+            
+            if( scalar(keys %{$cvpdnSystemTunnelTotal}) > 0 )
             {
                 $devdetails->setCap('ciscoVPDN');
 
-                $devdetails->storeSnmpVars( $tableTun );
-
                 # VPDN indexing: 1: l2f, 2: l2tp, 3: pptp
-                foreach my $typeIndex (
-                    $devdetails->getSnmpIndices(
-                        $dd->oiddef('cvpdnSystemTunnelTotal') ) )
+                foreach my $typeIndex
+                    (keys %{$cvpdnSystemTunnelTotal})
                 {
                     Debug("CISCO-VPDN-MGMT-MIB: found Tunnel type " .
                           $tunnelType{$typeIndex} );
-
+                    
                     $data->{'ciscoVPDN'}{$typeIndex} = $tunnelType{$typeIndex};
                 }
             }
         }
     }
 
-    if( $devdetails->param('CiscoIOS::disable-3g-stats') ne 'yes' )
+    if( $devdetails->paramDisabled('CiscoIOS::disable-3g-stats') )
     {
-        my $base = $dd->oiddef('c3gStandard');
-        my $table = $session->get_table( -baseoid => $base );
-        my $prefixLen = length( $base ) + 1;
-        
-        if( defined( $table ) and scalar( %{$table} ) > 0 )
-        {            
-            while( my( $oid, $val ) = each %{$table} )
+        my $table = $dd->walkSnmpTable('c3gStandard');        
+        while( my( $phy, $val ) = each %{$table} )
+        {
+            if( $val == 2 )
             {
-                if( $val == 2 )
+                $devdetails->setCap('Cisco3G');
+                my $phyName = $data->{'entityPhysical'}{$phy}{'name'};
+                if( not defined( $phyName ) )
                 {
-                    $devdetails->setCap('Cisco3G');
-                    my $phy = substr( $oid, $prefixLen );
-                    my $phyName = $data->{'entityPhysical'}{$phy}{'name'};
-                    if( not defined( $phyName ) )
-                    {
-                        $phyName = 'Cellular Modem';
-                    }
-                    
-                    $data->{'cisco3G'}{$phy} = {
-                        'name'   => $phyName,
-                        'standard' => 'gsm',
-                    };
-                }
-                else
-                {
-                    Warn('CISCO-WAN-3G-MIB: CDMA statistics are ' .
-                         'not yet implemented');
+                    $phyName = 'Cellular Modem';
                 }
+                
+                $data->{'cisco3G'}{$phy} = {
+                    'name'   => $phyName,
+                    'standard' => 'gsm',
+                };
+            }
+            else
+            {
+                Warn('CISCO-WAN-3G-MIB: CDMA statistics are ' .
+                     'not yet implemented');
             }
         }
 
-        my $tokenset = $devdetails->param('CiscoIOS::3g-stats-tokenset');
-        if( defined($tokenset) )
+        my $tokenset = $devdetails->paramString('CiscoIOS::3g-stats-tokenset');
+        if( $tokenset ne '')
         {
             $data->{'cisco3G_tokenset'} = $tokenset;
         }
     }
 
 
-    if( $devdetails->param('CiscoIOS::disable-port-qos-stats') ne 'yes' )
+    if( $devdetails->paramDisabled('CiscoIOS::disable-port-qos-stats') )
     {
-        my $base = $dd->oiddef('cportQosDropPkts');
-        my $table = $session->get_table( -baseoid => $base );
-        my $prefixLen = length( $base ) + 1;
-        
-        if( defined( $table ) and scalar( %{$table} ) > 0 )
+        my $table = $dd->walkSnmpTable('cportQosDropPkts');
+        if( scalar(keys %{$table}) > 0 )
         {            
             $devdetails->setCap('CiscoPortQos');
             $data->{'ciscoPortQos'} = {};
             
-            while( my( $oid, $val ) = each %{$table} )
+            foreach my $qos_entry (keys %{$table})
             {
-                my $qos_entry = substr( $oid, $prefixLen );
                 my($ifIndex, $direction, $qos_index) =
                     split(/\./o, $qos_entry);
                 
@@ -618,12 +556,14 @@ sub discover
     }
     
     
-    if( $devdetails->param('CiscoIOS::short-device-comment') eq 'yes' )
+    if( $devdetails->paramEnabled('CiscoIOS::short-device-comment')
+        and
+        defined($data->{'param'}{'comment'}) )
     {
         # Remove serials from device comment
         # 1841 chassis, Hw Serial#: 3625140487, Hw Revision: 6.0
         
-        $data->{'param'}{'comment'} =~ s/, Hw.*//o;        
+        $data->{'param'}{'comment'} =~ s/, Hw.*//o;
     }
 
     return 1;
@@ -648,7 +588,7 @@ sub buildConfig
                              } );
 
         foreach my $INDEX ( sort
-                            { $data->{'cbgpPeers'}{$a}{'subtreeName'} <=>
+                            { $data->{'cbgpPeers'}{$a}{'subtreeName'} cmp
                                   $data->{'cbgpPeers'}{$b}{'subtreeName'} }
                             keys %{$data->{'cbgpPeers'}} )
         {
@@ -698,39 +638,64 @@ sub buildConfig
                 $subtreeName .= '_' . $car->{'carIndex'};
             }
             
-            my $param = {
+            my $ifParam = {
                 'searchable' => 'yes',
                 'car-direction' => $car->{'direction'},
                 'car-index' => $car->{'carIndex'} };
             
-            $param->{'interface-name'} =
-                $interface->{'param'}{'interface-name'};            
-            $param->{'interface-nick'} =
-                $interface->{'param'}{'interface-nick'};            
-            $param->{'comment'} =
-                $interface->{'param'}{'comment'};
+            $ifParam->{'interface-name'} =
+                $interface->{$data->{'nameref'}{'ifReferenceName'}};
+            $ifParam->{'interface-nick'} =
+                $interface->{$data->{'nameref'}{'ifNick'}};
+            $ifParam->{'node-display-name'} =
+                $interface->{$data->{'nameref'}{'ifReferenceName'}};
+            
+            if( defined($data->{'nameref'}{'ifComment'}) and
+                defined($interface->{$data->{'nameref'}{'ifComment'}}) )
+            {
+                $ifParam->{'comment'} =
+                    $interface->{$data->{'nameref'}{'ifComment'}};
+            }
 
             my $legend = sprintf("Type: %s;", $car->{'configType'});
             if( $car->{'accIdx'} > 0 )
             {
                 $legend .= sprintf("Access list: %d;", $car->{'accIdx'});
             }
-            
-            $legend .=
-                sprintf("Rate: %d bps; Limit: %d bytes; Ext limit: %d bytes;" .
-                        "Conform action: %s; Exceed action: %s",
-                        $car->{'rate'},
-                        $car->{'limit'},
-                        $car->{'extLimit'},
-                        $car->{'conformAction'},
-                        $car->{'exceedAction'});
 
-            $param->{'legend'} = $legend;
+            if( defined($car->{'rate'}) )
+            {
+                $legend .= sprintf('Rate: %d bps;', $car->{'rate'});
+            }
+            
+            if( defined($car->{'limit'}) )
+            {
+                $legend .= sprintf('Limit: %d bytes;', $car->{'limit'});
+            }
+
+            if( defined($car->{'extLimit'}) )
+            {
+                $legend .= sprintf('Ext limit: %d bytes;', $car->{'extLimit'});
+            }
+
+            if( defined($car->{'conformAction'}) )
+            {
+                $legend .= sprintf('Conform action: %s;',
+                                   $car->{'conformAction'});
+            }
+
+            if( defined($car->{'exceedAction'}) )
+            {
+                $legend .= sprintf('Exceed action: %s',
+                                   $car->{'exceedAction'});
+            }
+
+            $ifParam->{'legend'} = $legend;
 
             $cb->addSubtree
                 ( $countersNode,
                   $subtreeName,
-                  $param, 
+                  $ifParam, 
                   ['CiscoIOS::cisco-car']);
         }
     }
@@ -740,7 +705,8 @@ sub buildConfig
     { 
         my $tunnelNode = $cb->addSubtree
             ( $devNode, 'VPDN_Statistics',
-              {'node-display-name' => 'VPDN Statistics'},
+              {'node-display-name' => 'VPDN Statistics',
+               'comment' => 'VPDN tunnel and session counters'},
               [ 'CiscoIOS::cisco-vpdn-subtree' ] );
 
         foreach my $INDEX ( sort keys %{$data->{'ciscoVPDN'}} )
@@ -837,6 +803,8 @@ sub buildConfig
                 ['CiscoIOS::cisco-port-qos-packets']);
         }
     }
+
+    return;
 }
 
 
