@@ -14,7 +14,6 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # DOCSIS interface, Cisco specific
@@ -22,6 +21,8 @@
 package Torrus::DevDiscover::CiscoIOS_Docsis;
 
 use strict;
+use warnings;
+
 use Torrus::Log;
 
 # Sequence number is 600 - we depend on RFC2670_DOCS_IF and CiscoIOS
@@ -272,6 +273,8 @@ sub buildConfig
 
         $cb->addParams( $upstrNode, $param );
     }
+
+    return;
 }
 
 
