@@ -14,7 +14,6 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Cisco IOS Service Assurance Agent
@@ -27,8 +26,9 @@
 package Torrus::DevDiscover::CiscoIOS_SAA;
 
 use strict;
-use Socket qw(inet_ntoa);
+use warnings;
 
+use Socket qw(inet_ntoa);
 use Torrus::Log;
 
 
@@ -178,7 +178,7 @@ sub checkdevtype
         my $rttAdminTable =
             $session->get_table( -baseoid =>
                                  $dd->oiddef('rttMonCtrlAdminTable') );
-        if( defined $rttAdminTable and scalar( %{$rttAdminTable} ) > 0 )
+        if( defined($rttAdminTable) and scalar(keys %{$rttAdminTable}) > 0 )
         {
             $devdetails->storeSnmpVars( $rttAdminTable );
             return 1;
@@ -298,6 +298,8 @@ sub buildConfig
         $cb->addSubtree( $subtreeNode, $subtreeName, $param,
                          ['CiscoIOS_SAA::cisco-rtt-echo-subtree']);
     }
+
+    return;
 }
 
 
