@@ -1,4 +1,4 @@
-#  Copyright (C) 2003 Shawn Ferry
+#  Copyright (C) 2003-2012 Shawn Ferry, Roman Hochuli
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -15,17 +15,13 @@
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
 # Shawn Ferry <sferry at sevenspace dot com> <lalartu at obscure dot org>
-
-# there are lots of errors, but we cannot re-test the module yet.
-## no critic (Variables::ProhibitReusedNames)
+# Roman Hochuli <roman dot hochuli at nexellent dot ch> <roman at hochu dot li>
 
 package Torrus::DevDiscover::EmpireSystemedge;
 
 use strict;
-use warnings;
-
 use Torrus::Log;
-
+use Data::Dumper;
 
 $Torrus::DevDiscover::registry{'EmpireSystemedge'} = {
     'sequence'     => 500,
@@ -93,6 +89,18 @@ our %oiddef =
      'empireMon'                => '1.3.6.1.4.1.546.6.1.1',
      'empirePmon'               => '1.3.6.1.4.1.546.15.1.1',
      'empireLog'                => '1.3.6.1.4.1.546.11.1.1',
+     
+     # Empire Service Response Extension
+     'empireSvcTable'           => '1.3.6.1.4.1.546.16.6.10.1',
+     'empireSvcIndex'           => '1.3.6.1.4.1.546.16.6.10.1.1',
+     'empireSvcDescr'           => '1.3.6.1.4.1.546.16.6.10.1.2',
+     'empireSvcType'            => '1.3.6.1.4.1.546.16.6.10.1.3',
+     'empireSvcTotRespTime'     => '1.3.6.1.4.1.546.16.6.10.1.12',
+     'empireSvcAvailability'    => '1.3.6.1.4.1.546.16.6.10.1.17',
+     'empireSvcConnTime'        => '1.3.6.1.4.1.546.16.6.10.1.23',
+     'empireSvcTransTime'       => '1.3.6.1.4.1.546.16.6.10.1.28',
+     'empireSvcThroughput'      => '1.3.6.1.4.1.546.16.6.10.1.37',
+     'empireSvcDestination'     => '1.3.6.1.4.1.546.16.6.10.1.45',
      );
 
 our %storageDescTranslate =  ( '/' => {'subtree' => 'root' } );
@@ -153,13 +161,23 @@ our %osTranslate =
      22 => { 'name' => 'nt50Intel',    'ident' => 'nt',   'template' => 1, },
      23 => { 'name' => 'nt50Alpha',    'ident' => 'nt',   'template' => 0, },
      25 => { 'name' => 'aix5RS6000',   'ident' => 'unix', 'template' => 1, },
-     26 => { 'name' => 'nt52Intel',    'ident' => 'nt',   'template' => 0, },
+     26 => { 'name' => 'nt52Intel',    'ident' => 'nt',   'template' => 0, }, # linuxIA64
+     27 => { 'name' => 'linuxIntel',   'ident' => 'unix', 'template' => 1, },
+     28 => { 'name' => 'hpux11IA64',   'ident' => 'unix', 'template' => 0, }, # nt52IA64 Windows 2003 Itanium
+     29 => { 'name' => 'nt50Intel',    'ident' => 'nt',   'template' => 1, }, # nt52X64  Windows 2003 x64 (AMD64 or EMT64)
+     30 => { 'name' => 'nt50Intel',    'ident' => 'nt',   'template' => 1, },
+     31 => { 'name' => 'linuxIntel',   'ident' => 'unix', 'template' => 1, }, # nt52IA64 Windows 2003 Itanium
+     35 => { 'name' => 'nt50Intel',    'ident' => 'nt',   'template' => 1, },
      );
 
 # Solaris Virtual Interface Filtering
 our $interfaceFilter;
 my %solarisVirtualInterfaceFilter;
+my %winNTInterfaceFilter;
 
+# Key is some unique symbolic name, does not mean anything
+# ifType is the number to match the interface type
+# ifDescr is the regexp to match the interface description
 %solarisVirtualInterfaceFilter = (
     'Virtual Interface (iana 62)' => {
         'ifType'    =>  62,             # Obsoleted
@@ -174,6 +192,42 @@ my %solarisVirtualInterfaceFilter;
         },
     );
 
+
+# Key is some unique symbolic name, does not mean anything
+# ifType is the number to match the interface type
+# ifDescr is the regexp to match the interface description
+# shameless rip-off from MicrsoftWindows.pm
+%winNTInterfaceFilter =
+    (
+     'MS TCP Loopback interface' => {
+         'ifType'  => 24                        # softwareLoopback
+         },
+     
+     'Tunnel' => {
+         'ifType'  => 131                       # tunnel
+         },
+     
+     'PPP' => {
+         'ifType'  => 23                        # ppp
+         },
+     
+     'WAN Miniport Ethernet' => {
+         'ifType'  => 6,                        # ethernetCsmacd
+         'ifDescr' => '^WAN[-|\s+]Miniport'
+         },
+     
+     'QoS Packet Scheduler' => {
+         'ifType'  => 6,                        # ethernetCsmacd
+         'ifDescr' => 'QoS\s+Packet\s+Scheduler'
+         },
+
+     'LightWeight Filter' => {
+         'ifType'  => 6,                        # ethernetCsmacd
+         'ifDescr' => 'WFP\s+LightWeight\s+Filter'
+         },
+     );
+ 
+
 our $storageGraphTop;
 our $storageHiMark;
 our $shortTemplate;
@@ -192,8 +246,7 @@ sub checkdevtype
         return 0;
     }
 
-    my $result = $dd->retrieveSnmpOIDs( 'sysedge_opmode',
-                                        'empireSystemType' );
+    my $result = $dd->retrieveSnmpOIDs( 'sysedge_opmode' );
     if( $result->{'sysedge_opmode'} == 2 )
     {
         Error("Sysedge Agent NOT Licensed");
@@ -201,23 +254,26 @@ sub checkdevtype
     }
 
     # Empire OS Type (Needed here for interface filtering)
-    
-    my $empireOsType = $result->{'empireSystemType'};
-    if( defined($empireOsType) and $empireOsType > 0 )
+    my $empireOsType =
+        $session->get_request( -varbindlist =>
+                               [ $dd->oiddef('empireSystemType') ] );
+
+    if( $session->error_status() == 0 )
     {
-        $devdetails->setCap('EmpireSystemedge::' .
-                            $osTranslate{$empireOsType}{ident} );
-        
-        $devdetails->{'os_ident'} = $osTranslate{$empireOsType}{ident};
 
-        
-        $devdetails->setCap('EmpireSystemedge::' .
-                            $osTranslate{$empireOsType}{name} );
-        
-        $devdetails->{'os_name'} = $osTranslate{$empireOsType}{name};
+        $devdetails->setCap('EmpireSystemedge::' . $osTranslate{
+            $empireOsType->{$dd->oiddef('empireSystemType')} }{ident} );
+        $devdetails->{'os_ident'} = $osTranslate{
+            $empireOsType->{$dd->oiddef('empireSystemType')} }{ident};
+
+        $devdetails->setCap('EmpireSystemedge::' . $osTranslate{
+            $empireOsType->{$dd->oiddef('empireSystemType')} }{name} );
+        $devdetails->{'os_name'} = $osTranslate{
+            $empireOsType->{$dd->oiddef('empireSystemType')} }{name};
+
+        $devdetails->{'os_name_template'} = $osTranslate{
+            $empireOsType->{$dd->oiddef('empireSystemType')} }{template};
 
-        $devdetails->{'os_name_template'} =
-            $osTranslate{$empireOsType}{template};
     }
 
     # Exclude Virtual Interfaces on Solaris
@@ -228,6 +284,16 @@ sub checkdevtype
             ($devdetails, $interfaceFilter);
     }
 
+    # Exclude strange interfaces on Windows
+    # shameless rip-off from MicrsoftWindows.pm
+    if( ( $devdetails->{'os_name'} =~ /nt40/i ) or ( $devdetails->{'os_name'} =~ /nt50/i )) {
+
+        $interfaceFilter = \%winNTInterfaceFilter;
+        &Torrus::DevDiscover::RFC2863_IF_MIB::addInterfaceFilter
+            ($devdetails, $interfaceFilter);
+        $devdetails->setCap('interfaceIndexingManaged');
+    }
+
     return 1;
 }
 
@@ -237,278 +303,272 @@ sub discover
     my $dd = shift;
     my $devdetails = shift;
 
-    my $data = $devdetails->data();
     my $session = $dd->session();
+    my $data = $devdetails->data();
 
 
-    if( $dd->checkSnmpOID('empireCpuTotalWait') )
-    {
+    # Exclude strange interfaces on Windows
+    # shameless rip-off from MicrsoftWindows.pm
+    if( ( $devdetails->{'os_name'} =~ /nt40Intel/i ) or ( $devdetails->{'os_name'} =~ /nt50Intel/i )) {
+
+        $data->{'nameref'}{'ifComment'} = ''; # suggest?
+        $data->{'param'}{'ifindex-map'} = '$IFIDX_MAC';
+        Torrus::DevDiscover::RFC2863_IF_MIB::retrieveMacAddresses( $dd, $devdetails );
+        $data->{'nameref'}{'ifNick'} = 'MAC';
+        $data->{'nameref'}{'ifNodeid'} = 'MAC';    
+    }
+
+
+    # Empire Cpu Totals
+
+    if($dd->checkSnmpTable('empireCpuTotalWait')) {
         $devdetails->setCap('EmpireSystemedge::CpuTotal::Wait');
     }
 
+
+    # The layout of these discover-procedures look somewhat nuts but walking the tables first and testing later if it returns
+    # values uses one less snmp-request than checking for the existance of the tables first and walking them again later. 
+    # This brings a whopping ~0.0090s/device faster discovery. See discussion here:
+    # https://github.com/medea61/torrus-newfeatures/commit/ba958bd27011243a3daa55c8bd657d4c8bf9d04c#commitcomment-1702848
+    
     # Empire Dev Stats Table
 
-    my $empireDiskStatsTable =
-        $session->get_table( -baseoid =>
-                             $dd->oiddef('empireDiskStatsTable') );
-
-    my $hrDeviceDescr = $session->get_table( -baseoid =>
-                                             $dd->oiddef('hrDeviceDescr') );
-
-    if( defined($empireDiskStatsTable) and defined($hrDeviceDescr) )
     {
-        $devdetails->setCap('EmpireSystemedge::DiskStats');
-        $devdetails->storeSnmpVars( $empireDiskStatsTable );
-        $devdetails->storeSnmpVars( $hrDeviceDescr );
+        my $indices = $dd->walkSnmpTable('empireDiskStatsIndex');
+        my $hrtable = $dd->walkSnmpTable('hrDeviceDescr');
 
-        my $ref= {'indices' => []};
-        $data->{'empireDiskStats'} = $ref;
+        if((scalar(keys %{$indices}) > 0) && (scalar(keys %{$hrtable}) > 0)) {
+            my $dshostindex = $dd->walkSnmpTable('empireDiskStatsHostIndex');
 
-        foreach my $INDEX
-            ( $devdetails->
-              getSnmpIndices( $dd->oiddef('empireDiskStatsIndex') ) )
-        {
-            next if( $INDEX < 1 );
-
-            my $hrindex =
-                $devdetails->snmpVar( $dd->oiddef('empireDiskStatsHostIndex') .
-                                      '.' . $INDEX );
-
-            next if( $hrindex < 1 );
-
-            push( @{ $ref->{'indices'} }, $INDEX );
-
-            my $descr = $devdetails->snmpVar($dd->oiddef('hrDeviceDescr') .
-                                             '.' . $hrindex );
-
-            my $ref = { 'param' => {}, 'templates' => [] };
-            $data->{'empireDiskStats'}{$INDEX} = $ref;
-            my $param = $ref->{'param'};
+            $devdetails->setCap('EmpireSystemedge::DiskStats');
+            $data->{'empireDiskStats'} = {};
+            $data->{'empireDiskStats'}{'indices'} = [];
 
 
-            $param->{'comment'} = $descr;
+            while( my( $index, $value ) = each %{$indices} ) {
+                push(@{$data->{'empireDiskStats'}{'indices'}}, $index);
 
-            $param->{'HRINDEX'} = $hrindex;
+                $data->{'empireDiskStats'}{$index}{'templates'} = [];
+                $data->{'empireDiskStats'}{$index}{'param'}{'HRINDEX'} = $dshostindex->{$index};
+                $data->{'empireDiskStats'}{$index}{'param'}{'comment'} = $hrtable->{$dshostindex->{$index}};
 
-            if ( not defined $descr )
-            {
-                $descr = "Index $hrindex";
+                if(not defined($hrtable->{$dshostindex->{$index}})) {
+                    $data->{'empireDiskStats'}{$index}{'param'}{'disk-stats-description'} = 'Index ' . $dshostindex->{$index};
+                } else {
+                    $data->{'empireDiskStats'}{$index}{'param'}{'disk-stats-description'} = $hrtable->{$dshostindex->{$index}};                
+                }
+
+                my $nick = $hrtable->{$dshostindex->{$index}};
+                $nick =~ s/^\///;
+                $nick =~ s/\W/_/g;
+                $data->{'empireDiskStats'}{$index}{'param'}{'disk-stats-nick'} = $nick;
             }
-            $param->{'disk-stats-description'} = $descr;
-
-            $descr =~ s/^\///;
-            $descr =~ s/\W/_/g;
-            $param->{'disk-stats-nick'} = $descr;
-
         }
-    } # end empireDiskStatsTable
+    }
+
 
     # Empire Dev Table
 
-    my $empireDevTable = $session->get_table( -baseoid =>
-                                              $dd->oiddef('empireDevTable') );
-
-    if( defined( $empireDevTable ) )
     {
+        my $indices = $dd->walkSnmpTable('empireDevIndex');
 
-        $devdetails->setCap('EmpireSystemedge::Devices');
-        $devdetails->storeSnmpVars( $empireDevTable );
+        if(scalar(keys %{$indices}) > 0) {
+            my $types = $dd->walkSnmpTable('empireDevType');
+            my $descr = $dd->walkSnmpTable('empireDevMntPt');
+            my $bsize = $dd->walkSnmpTable('empireDevBsize');
+            my $device = $dd->walkSnmpTable('empireDevDevice');
+            my $size = $dd->walkSnmpTable('empireDevTblks');
 
-        my $ref= {};
-        $data->{'empireDev'} = $ref;
+            $devdetails->setCap('EmpireSystemedge::Devices');
+            $data->{'empireDev'} = {};
+            $data->{'empireDev'}{'indices'} = [];
 
-        foreach my $INDEX
-            ( $devdetails->getSnmpIndices($dd->oiddef('empireDevIndex') ) )
-        {
-            next if( $INDEX < 1 );
+            while( my( $index, $value ) = each %{$indices} ) {
+                if ($bsize->{$index} and defined ($descr->{$index})) {
+                    push(@{$data->{'empireDev'}{'indices'}}, $index);
 
+                    $data->{'empireDev'}{$index}{'templates'} = [];
+                    $data->{'empireDev'}{$index}{'param'}{'storage-description'} = $descr->{$index};
+                    $data->{'empireDev'}{$index}{'param'}{'storage-device'} = $device->{$index};
+                    $data->{'empireDev'}{$index}{'param'}{'node-display-name'} = $device->{$index};
 
-            my $type = $devdetails->snmpVar( $dd->oiddef('empireDevType') .
-                                             '.' . $INDEX );
-
-            my $descr = $devdetails->snmpVar($dd->oiddef('empireDevMntPt') .
-                                             '.' . $INDEX );
-
-            my $bsize =  $devdetails->snmpVar($dd->oiddef('empireDevBsize') .
-                                              '.' . $INDEX );
-
-            # NFS has a block size of 0, it will be skipped
-            if( $bsize and defined( $descr ) )
-            {
-                push( @{ $data->{'empireDev'}->{'indices'} }, $INDEX);
-
-                my $ref = { 'param' => {}, 'templates' => [] };
-                $data->{'empireDev'}{$INDEX} = $ref;
-                my $param = $ref->{'param'};
-
-                $param->{'storage-description'} = $descr;
-                $param->{'storage-device'} =
-                    $devdetails->snmpVar($dd->oiddef('empireDevDevice')
-                                         . '.' . $INDEX );
-
-                my $comment = $type;
-                if( $descr =~ /^\// )
-                {
-                    $comment .= ' (' . $descr . ')';
-                }
-                $param->{'comment'} = $comment;
-
-                if( $storageDescTranslate{$descr}{'subtree'} )
-                {
-                    $descr = $storageDescTranslate{$descr}{'subtree'};
-                }
-                $descr =~ s/^\///;
-                $descr =~ s/\W/_/g;
-                $param->{'storage-nick'} = $descr;
-
-                my $units = $bsize;
-
-                $param->{'collector-scale'} = sprintf('%d,*', $units);
-
-                my $size =
-                    $devdetails->snmpVar
-                    ($dd->oiddef('empireDevTblks') . '.' . $INDEX);
-
-                if( $size )
-                {
-                    if( $storageGraphTop > 0 )
+                    my $comment = $types->{$index};
+                    if( $descr->{$index} =~ /^\// )
                     {
-                        $param->{'graph-upper-limit'} =
-                            sprintf('%e',
-                                    $units * $size * $storageGraphTop / 100 );
+                        $comment .= ' (' . $descr->{$index} . ')';
+                    }
+                    $data->{'empireDev'}{$index}{'param'}{'comment'} = $comment;
+
+                    my $devdescr = $descr->{$index};
+                    if( $storageDescTranslate{$descr->{$index}}{'subtree'} )
+                    {
+                        $devdescr = $storageDescTranslate{$descr->{$index}}{'subtree'};
+                    }
+                    $devdescr =~ s/^\///;
+                    $devdescr =~ s/\W/_/g;
+                    $data->{'empireDev'}{$index}{'param'}{'storage-nick'} = $devdescr;
+
+                    my $units = $bsize->{$index};
+                    $data->{'empireDev'}{$index}{'param'}{'collector-scale'} = sprintf('%d,*', $units);
+
+                    if($size->{$index}) {
+                        if($storageGraphTop > 0)
+                        {
+                            $data->{'empireDev'}{$index}{'param'}{'graph-upper-limit'} = sprintf('%e', $units * $size->{$index} * $storageGraphTop / 100 );
+                        }
+
+                        if( $storageHiMark > 0 )
+                        {
+                            $data->{'empireDev'}{$index}{'param'}{'upper-limit'} = sprintf('%e', $units * $size->{$index} * $storageHiMark / 100 );
+                        }
                     }
 
-                    if( $storageHiMark > 0 )
-                    {
-                        $param->{'upper-limit'} =
-                            sprintf('%e',
-                                    $units * $size * $storageHiMark / 100 );
-                    }
                 }
-
             }
+
+            $devdetails->clearCap('hrStorage');
         }
-
-        $devdetails->clearCap( 'hrStorage' );
-
-    } # end empireDevTable
+    }
 
 
     # Empire Per - Cpu Table
 
-    my $empireCpuStatsTable =
-        $session->get_table( -baseoid =>
-                             $dd->oiddef('empireCpuStatsTable') );
-
-    if( defined( $empireCpuStatsTable ) )
     {
-        $devdetails->setCap('EmpireSystemedge::CpuStats');
-        $devdetails->storeSnmpVars( $empireCpuStatsTable );
+        my $indices = $dd->walkSnmpTable('empireCpuStatsIndex');
 
-        my $ref= {};
-        $data->{'empireCpuStats'} = $ref;
+        if(scalar(keys %{$indices}) > 0) {
+            my $table = $dd->walkSnmpTable('empireCpuStatsTable');
+            my $cpuStatsDescr = 2;
 
-        foreach my $INDEX
-            ( $devdetails->
-              getSnmpIndices( $dd->oiddef('empireCpuStatsIndex') ) )
-        {
-            next if( $INDEX < 1 );
+            $devdetails->setCap('EmpireSystemedge::CpuStats');
+            $data->{'empireCpuStats'} = {};
+            $data->{'empireCpuStats'}{'indices'} = [];
 
-            push( @{ $ref->{'indices'} }, $INDEX);
+            while( my( $index, $value ) = each %{$indices} ) {
+                push(@{$data->{'empireCpuStats'}{'indices'}}, $index);
 
-            my $descr =
-                $devdetails->snmpVar( $dd->oiddef('empireCpuStatsDescr') .
-                                      '.' . $INDEX );
+                $data->{'empireCpuStats'}{$index}{'templates'} = [];
+                $data->{'empireCpuStats'}{$index}{'param'}{'INDEX'} = $index;
+                $data->{'empireCpuStats'}{$index}{'param'}{'cpu'} = 'CPU' . $index;
+                $data->{'empireCpuStats'}{$index}{'param'}{'descr'} = $table->{$cpuStatsDescr . '.' . $index};
+                $data->{'empireCpuStats'}{$index}{'param'}{'comment'} = $data->{'empireCpuStats'}{$index}{'param'}{'descr'} . ' (' . 'CPU ' . $index . ')';
 
-            my $ref = { 'param' => {}, 'templates' => [] };
-            $data->{'empireCpuStats'}{$INDEX} = $ref;
-            my $param = $ref->{'param'};
-
-            $param->{'cpu'} = 'CPU' . $INDEX;
-            $param->{'descr'} = $descr;
-            $param->{'INDEX'} = $INDEX;
-            $param->{'comment'} = $descr . ' (' . 'CPU ' . $INDEX . ')';
+            }
         }
     }
 
+
     # Empire Load Average
 
-    if( $dd->checkSnmpOID('empireLoadAverage') )
-    {
+    if($dd->checkSnmpTable('empireLoadAverage')) {
         $devdetails->setCap('EmpireSystemedge::LoadAverage');
+        my $ref = {'indices' => []};
+        $data->{'empireLoadAverage'} = $ref;
     }
+    
 
     # Empire Swap Counters
 
-    if( $dd->checkSnmpOID('empireNumPageSwapIns') )
-    {
+    if($dd->checkSnmpTable('empireNumPageSwapIns')) {
         $devdetails->setCap('EmpireSystemedge::SwapCounters');
     }
+    
 
     # Empire Counter Traps
 
-    if( $dd->checkSnmpOID('empireNumTraps') )
-    {
+    if($dd->checkSnmpTable('empireNumTraps')) {
         $devdetails->setCap('EmpireSystemedge::CounterTraps');
     }
+    
 
     # Empire Performance
 
-    my $empirePerformance =
-        $session->get_table( -baseoid => $dd->oiddef('empirePerformance') );
-
-    if( defined( $empirePerformance ) )
-    {
+    if($dd->checkSnmpTable('empirePerformance')) {
         $devdetails->setCap('EmpireSystemedge::Performance');
-        $devdetails->storeSnmpVars( $empirePerformance );
 
-        if( defined $devdetails->snmpVar($dd->oiddef('empireRunq') ) )
+        if($dd->checkSnmpTable('empireRunq'))
         {
             $devdetails->setCap('EmpireSystemedge::RunQ');
         }
-
-        if( defined $devdetails->snmpVar($dd->oiddef('empireDiskWait') ) )
+        
+        if($dd->checkSnmpTable('empireDiskWait'))
         {
             $devdetails->setCap('EmpireSystemedge::DiskWait');
         }
-
-        if( defined $devdetails->snmpVar($dd->oiddef('empirePageWait') ) )
+        
+        if($dd->checkSnmpTable('empirePageWait'))
         {
             $devdetails->setCap('EmpireSystemedge::PageWait');
         }
-
-        if( defined $devdetails->snmpVar($dd->oiddef('empireSwapActive') ) )
+        
+        if($dd->checkSnmpTable('empireSwapActive'))
         {
             $devdetails->setCap('EmpireSystemedge::SwapActive');
         }
-
-        if( defined $devdetails->snmpVar($dd->oiddef('empireSleepActive') ) )
+        
+        if($dd->checkSnmpTable('empireSleepActive'))
         {
             $devdetails->setCap('EmpireSystemedge::SleepActive');
         }
     }
 
-    my $empireNTREGPERF =
-        $session->get_table( -baseoid => $dd->oiddef('empireNTREGPERF') );
-    if( defined $empireNTREGPERF )
+
+    # Empire Service Checks
+
     {
-        $devdetails->setCap('empireNTREGPERF');
-        $devdetails->storeSnmpVars( $empireNTREGPERF );
+        my $indices = $dd->walkSnmpTable('empireSvcIndex');
 
-        my $ref = {};
-        $data->{'empireNTREGPERF'} = $ref;
-        foreach my $INDEX
-            ( $devdetails->getSnmpIndices($dd->oiddef('empireNTREGPERF') ) )
-        {
-            # This is all configured on a per site basis.
-            # The xml will be site specific
-            push( @{ $ref->{'indices'} }, $INDEX);
-            my $template = {};
-            $Torrus::ConfigBuilder::templateRegistry->
-            {'EmpireSystemedge::NTREGPERF_' . $INDEX} = $template;
-            $template->{'name'}='EmpireSystemedge::NTREGPERF_' . $INDEX;
-            $template->{'source'}='vendor/empire.systemedge.ntregperf.xml';
+        if(scalar(keys %{$indices}) > 0) {
+            my $table = $dd->walkSnmpTable('empireSvcTable');
+            my $svcDescr = 2;
+            my $svcType = 3;
+            my $svcTotRespTime = 12;
 
+            $devdetails->setCap('EmpireSystemedge::ServiceResponse');
+            $data->{'empireSvcStats'} = {};
+            $data->{'empireSvcStats'}{'indices'} = [];
+
+
+            while( my( $index, $value ) = each %{$indices} ) {
+                push(@{$data->{'empireSvcStats'}{'indices'}}, $index);
+
+                if($table->{$svcType . '.' . $index} eq "4") {
+                    $data->{'empireSvcStats'}{$index}{'param'}{'INDEX'} = $index;
+                    $data->{'empireSvcStats'}{$index}{'param'}{'id'} = 'Responder_' . $index;
+                    $data->{'empireSvcStats'}{$index}{'param'}{'descr'} = $table->{$svcDescr . '.' . $index};
+                    $data->{'empireSvcStats'}{$index}{'param'}{'descr'} = $table->{$svcDescr . '.' . $index};
+
+                    if ( defined $devdetails->{'params'}->{'node-display-name'} ) {
+                        $data->{'empireSvcStats'}{$index}{'param'}{'name'} = $devdetails->{'params'}->{'node-display-name'};
+                    }
+                    elsif ( defined $devdetails->{'params'}->{'symbolic-name'} ) {
+                        $data->{'empireSvcStats'}{$index}{'param'}{'name'} = $devdetails->{'params'}->{'symbolic-name'};
+                    }
+                    else {
+                        $data->{'empireSvcStats'}{$index}{'param'}{'name'} = $data->{'param'}->{'snmp-host'};
+                    }
+                }
+            } 
+        }
+    }
+
+
+    # Empire NTREGPERF
+
+    {
+        my $indices = $dd->walkSnmpTable('empireNTREGPERF');
+        if(scalar(keys %{$indices}) > 0) {
+            $devdetails->setCap('EmpireSystemedge::empireNTREGPERF');
+            $data->{'empireNTREGPERF'} = {};
+            $data->{'empireNTREGPERF'}{'indices'} = [];
+
+            while( my( $index, $value ) = each %{$indices} ) {
+                push(@{$data->{'empireSvcStats'}{'indices'}}, $index);
+
+                $Torrus::ConfigBuilder::templateRegistry->{'EmpireSystemedge::NTREGPERF_' . $index} = {};
+                $Torrus::ConfigBuilder::templateRegistry->{'EmpireSystemedge::NTREGPERF_' . $index}{'name'}='EmpireSystemedge::NTREGPERF_' . $index;
+                $Torrus::ConfigBuilder::templateRegistry->{'EmpireSystemedge::NTREGPERF_' . $index}{'source'}='vendor/empire.systemedge.ntregperf.xml';
+            } 
         }
     }
 
@@ -628,6 +688,7 @@ sub buildConfig
         my $param = {
             'precedence'    => '-1000',
             'comment'       => 'Physical/Logical Volume Information',
+            'node-display-name' => 'Volume Information',
         };
 
         my $subtreeNode =
@@ -653,10 +714,29 @@ sub buildConfig
         my $subtreeName = "DiskInfo";
 
         my $param = {
-            'precedence'    => '-1000',
-            'comment'       => 'Physical/Logical Disk Information',
+            'precedence'        => '-1000',
+            'comment'           => 'Physical/Logical Disk Information',
+            'node-display-name' => 'Disk Information',
         };
 
+        # dynamically make overview-subleaves links visible
+        if ( $os_target =~ /nt40/i or $os_target =~ /nt50/i or $os_target =~ /solaris/i ) {
+            $param->{'has-overview-shortcuts'} = 'yes';
+            $param->{'overview-shortcuts'} = 'rw,util,qlen';
+        }
+        elsif ( $os_target =~ /aix/i ) {
+            $param->{'has-overview-shortcuts'} = 'yes';
+            $param->{'overview-shortcuts'} = 'rw,util';
+        }
+        elsif ( $os_target =~ /linux/i or $os_target =~ /unix/i ) {
+            $param->{'has-overview-shortcuts'} = 'yes';
+            $param->{'overview-shortcuts'} = 'rw';
+        }
+        else {
+            # basically we should never get down here...
+            $param->{'has-overview-shortcuts'} = 'no';
+        }
+
         my $subtreeNode =
             $cb->addSubtree( $StorageNode, $subtreeName, $param,
                              ['EmpireSystemedge::empire-disk-stats-subtree']);
@@ -672,14 +752,15 @@ sub buildConfig
                              $ref->{'param'},
                              [ 'EmpireSystemedge::empire-disk-stats-' .
                                $os_target, ] );
+
         }
     }
 
 
     # Performance Subtree
-    $subtreeName= "System_Performance";
+    my $subtreeName= "System_Performance";
 
-    $param = {
+    my $param = {
         'precedence'     => '-900',
         'comment'        => 'System, CPU and memory statistics'
         };
@@ -699,10 +780,10 @@ sub buildConfig
     }
 
     push( @perfTemplates,
-          'EmpireSystemedge::empire-counters-' . $os_target,
-          'EmpireSystemedge::empire-swap-counters-' . $os_target,
-          'EmpireSystemedge::empire-total-cpu-' .  $os_target,
-          'EmpireSystemedge::empire-total-cpu-raw-' .  $os_target,
+          'EmpireSystemedge::empire-counters-' . $devdetails->{'os_name'},
+          'EmpireSystemedge::empire-swap-counters-' . $devdetails->{'os_name'},
+          'EmpireSystemedge::empire-total-cpu-' .  $devdetails->{'os_name'},
+          'EmpireSystemedge::empire-total-cpu-raw-' .  $devdetails->{'os_name'},
           );
 
     if( $devdetails->hasCap('EmpireSystemedge::RunQ') )
@@ -766,9 +847,9 @@ sub buildConfig
         }
     }
 
-    if( $devdetails->hasCap('empireNTREGPERF') )
+    if( $devdetails->hasCap('EmpireSystemedge::empireNTREGPERF') )
     {
-        Debug("NTREGPERF");
+        Debug("EmpireSystemedge::NTREGPERF");
         my $ntregTree = "NT_REG_PERF";
         my $ntregParam = {
             'precedence'    => '-10000',
@@ -788,6 +869,41 @@ sub buildConfig
 
     }
 
+    if( $devdetails->hasCap('EmpireSystemedge::ServiceResponse') )
+    {
+        Debug("Empire SysEdge ServiceResponse");
+
+        my $ref = $data->{'empireSvcStats'};
+        
+        my $subtreeName = "Service_Response_Checks";
+        my $param = {
+            'precedence'        => '-1200',
+            'comment'           => 'Service Response Statistics',
+            'node-display-name' => 'Service Response Checks',
+        };
+
+        my $subtreeNode =
+            $cb->addSubtree( $devNode, $subtreeName, $param,
+                             [ 'EmpireSystemedge::empire-svc-subtree' ] );
+        
+        foreach my $INDEX
+            ( sort {$a<=>$b} @{$data->{'empireSvcStats'}{'indices'} } )
+        {
+            my $ref = $data->{'empireSvcStats'}{$INDEX};
+
+            # Display in index order
+            $ref->{'param'}->{'precedence'} = sprintf("%d", 1000 - $INDEX);
+            $ref->{'param'}->{'node-display-name'} = "Responder " . $ref->{'param'}{'descr'};
+            $ref->{'param'}->{'graph-title'} = $ref->{'param'}{'name'} . " - " . $ref->{'param'}{'descr'} . " - Responsetime";
+
+            $cb->addLeaf
+                ( $subtreeNode, $ref->{'param'}{'id'},
+                  $ref->{'param'},
+                  ['EmpireSystemedge::empire-svc-response'],
+                  );
+        }
+    }
+
     return;
 }
 
