@@ -15,7 +15,6 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Jon Nistor <nistor at snickers.org>
 
 # Juniper JunOS Discovery Module
@@ -37,6 +36,8 @@
 package Torrus::DevDiscover::JunOS;
 
 use strict;
+use warnings;
+
 use Torrus::Log;
 
 
@@ -179,19 +180,21 @@ sub discover
     my $chassisSerial =
         $dd->retrieveSnmpOIDs( 'jnxBoxDescr', 'jnxBoxSerialNo' );
 
-    if( defined( $chassisSerial ) )
+    if( not defined($data->{'param'}{'comment'}) )
     {
-        $data->{'param'}{'comment'} = $chassisSerial->{'jnxBoxDescr'} .
-            ', Hw Serial#: ' . $chassisSerial->{'jnxBoxSerialNo'};
-    } else
-    {
-        $data->{'param'}{'comment'} = "Juniper router";
+        if( defined( $chassisSerial ) )
+        {
+            $data->{'param'}{'comment'} = $chassisSerial->{'jnxBoxDescr'} .
+                ', Hw Serial#: ' . $chassisSerial->{'jnxBoxSerialNo'};
+        } else
+        {
+            $data->{'param'}{'comment'} = "Juniper router";
+        }
     }
 
-
     # PROG: Class of Service
     #
-    if( $devdetails->param('JunOS::disable-cos') ne 'yes' )
+    if( $devdetails->paramDisabled('JunOS::disable-cos') )
     {
         # Poll table to translate the CoS Index to a Name
         my $cosQueueNumTable =
@@ -239,7 +242,7 @@ sub discover
 
     # PROG: Grab and store description of parts
     #
-    if( $devdetails->param('JunOS::disable-operating') ne 'yes' )
+    if( $devdetails->paramDisabled('JunOS::disable-operating') )
     {
         my $tableDesc = $session->get_table( -baseoid =>
                                              $dd->oiddef('jnxOperatingDescr'));
@@ -304,7 +307,7 @@ sub discover
 
 
     # PROG: Firewall statistics
-    if( $devdetails->param('JunOS::disable-firewall') ne 'yes' )
+    if( $devdetails->paramDisabled('JunOS::disable-firewall') )
     {
         my $tableFWFilter =
             $session->get_table( -baseoid =>
@@ -357,7 +360,7 @@ sub discover
 
 
     # PROG: Check for RPF availability
-    if( $devdetails->param('JunOS::disable-rpf') ne 'yes' )
+    if( $devdetails->paramDisabled('JunOS::disable-rpf') )
     {
         my $tableRPF =
             $session->get_table( -baseoid =>
@@ -410,9 +413,9 @@ sub buildConfig
 
 
     # PROG: Class of Service information
-    if( $devdetails->hasCap('jnxCoS') &&
-	( keys %{$data->{'jnxCos'}{'ifIndex'}} > 0 )
-       )
+    if( $devdetails->hasCap('jnxCoS')
+        and
+	scalar(keys %{$data->{'jnxCos'}{'ifIndex'}}) > 0 )
     {
 	# PROG: Add CoS information if it exists.
 	my $nodeTop = $cb->addSubtree( $devNode, 'CoS', undef,
@@ -455,14 +458,14 @@ sub buildConfig
                                        'precedence' => 1000 - $cosIndex },
                                      [ 'JunOS::junos-cos-leaf' ]);
 
-                if( $devdetails->param('JunOS::disable-cos-tail') ne 'yes' )
+                if( $devdetails->paramDisabled('JunOS::disable-cos-tail') )
                 {
                     $cb->addSubtree( $nodeIFCOS, "Tail_drop_stats",
                                      { 'comment'  => 'Tail drop statistics' },
                                      [ 'JunOS::junos-cos-tail' ]);
                 }
 
-                if( $devdetails->param('JunOS::disable-cos-red') ne 'yes' )
+                if( $devdetails->paramDisabled('JunOS::disable-cos-red') )
                 {
                     $cb->addSubtree
                         ( $nodeIFCOS, "RED_stats",
@@ -645,6 +648,8 @@ sub buildConfig
             }
         }
     } # END: if jnxRPF
+
+    return;
 }
 
 
