@@ -14,7 +14,6 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Shawn Ferry <sferry at sevenspace dot com> <lalartu at obscure dot org>
 
 # NetScreen
@@ -22,6 +21,8 @@
 package Torrus::DevDiscover::NetScreen;
 
 use strict;
+use warnings;
+
 use Torrus::Log;
 
 
@@ -88,7 +89,7 @@ sub discover
     if( not defined( $data->{'param'}{'snmp-oids-per-pdu'} ) )
     {
         my $oidsPerPDU = $devdetails->param('NetScreen::snmp-oids-per-pdu');
-        if( $oidsPerPDU == 0 )
+        if( not defined($oidsPerPDU) or $oidsPerPDU == 0 )
         {
             $oidsPerPDU = 10;
         }
@@ -139,6 +140,8 @@ sub buildConfig
 
     $cb->addTemplateApplication($devNode, 'NetScreen::netscreen-cpu-stats');
     $cb->addTemplateApplication($devNode, 'NetScreen::netscreen-memory-stats');
+
+    return;
 }
 
 
