@@ -14,7 +14,6 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # DOCSIS interface statistics
@@ -22,6 +21,8 @@
 package Torrus::DevDiscover::RFC2670_DOCS_IF;
 
 use strict;
+use warnings;
+
 use Torrus::Log;
 
 
@@ -87,18 +88,13 @@ sub discover
         $devdetails->setCap('docsDownstreamUtil');
     }
 
-    my $snrTable =
-        $session->get_table( -baseoid =>
-                             $dd->oiddef('docsIfSigQSignalNoise') );
-    if( defined( $snrTable ) )
-    {
-        $devdetails->storeSnmpVars( $snrTable );
-    }
+    my $snrTable = $dd->walkSnmpTable('docsIfSigQSignalNoise');
     
     $data->{'docsCableMaclayer'} = [];
     $data->{'docsCableDownstream'} = [];
     $data->{'docsCableUpstream'} = [];
-
+    $data->{'docsCableUpstreamPhy'} = [];
+    
     foreach my $ifIndex ( sort {$a<=>$b} keys %{$data->{'interfaces'}} )
     {
         my $interface = $data->{'interfaces'}{$ifIndex};
@@ -127,23 +123,20 @@ sub discover
         }
         elsif( $ifType == 129 or $ifType == 205 )
         {
-            if( $devdetails->hasOID( $dd->oiddef('docsIfSigQSignalNoise') .
-                                     '.' . $ifIndex ) )
+            if( defined($snrTable->{$ifIndex}) )
             {
                 push( @{$data->{'docsCableUpstream'}}, $ifIndex );
                 push( @{$interface->{'docsTemplates'}},
                       'RFC2670_DOCS_IF::docsis-upstream-stats' );
                 
             }
+            else
+            {
+                push( @{$data->{'docsCableUpstreamPhy'}}, $ifIndex );
+            }
         }
     }
     
-    if( $devdetails->param('RFC2670_DOCS_IF::upstreams-only') eq 'yes' )
-    {
-        $data->{'docsCableMaclayer'} = [];
-        $data->{'docsCableDownstream'} = [];
-    }
-
     $data->{'docsConfig'} = {
         'docsCableMaclayer' => {
             'subtreeName' => 'Docsis_MAC_Layer',
@@ -171,6 +164,27 @@ sub discover
         },
     };
 
+    if( $devdetails->paramEnabled
+        ('RFC2670_DOCS_IF::suppress-all-cable-stats') )
+    {
+        # Completely exclude all DOCSIS interfaces from any collection
+        foreach my $category ( keys %{$data->{'docsConfig'}},
+                               'docsCableUpstreamPhy' )
+        {
+            foreach my $ifIndex ( @{$data->{$category}} )
+            {
+                my $interface = $data->{'interfaces'}{$ifIndex};
+                $interface->{'excluded'} = 1;
+            }
+        }
+    }
+
+    if( $devdetails->paramEnabled('RFC2670_DOCS_IF::upstreams-only') )
+    {
+        $data->{'docsCableMaclayer'} = [];
+        $data->{'docsCableDownstream'} = [];
+    }
+
     if( $devdetails->hasCap('docsDownstreamUtil') )
     {
         push( @{$data->{'docsConfig'}{'docsCableDownstream'}{'templates'}},
@@ -219,29 +233,29 @@ sub buildConfig
             {
                 my $interface = $data->{'interfaces'}{$ifIndex};
 
-                my $param = $interface->{'docsParams'};
+                my $ifParam = $interface->{'docsParams'};
 
-                $param->{'searchable'} = 'yes';
+                $ifParam->{'searchable'} = 'yes';
 
                 # Copy some parameters from IF-MIB discovery results
                 
                 foreach my $p ('interface-name', 'interface-nick',
                                'node-display-name', 'comment')
                 {
-                    $param->{$p} = $interface->{'param'}{$p};
+                    $ifParam->{$p} = $interface->{'param'}{$p};
                 }
                 
-                $param->{'nodeid-docsif'} =
+                $ifParam->{'nodeid-docsif'} =
                     $data->{'docsConfig'}{$category}{'nodeidCategory'} .
                     '//%nodeid-device%//' .
                     $interface->{$data->{'nameref'}{'ifNodeid'}};
                 
-                $param->{'nodeid'} = '%nodeid-docsif%';
+                $ifParam->{'nodeid'} = '%nodeid-docsif%';
                 
                 my $intfNode = $cb->addSubtree
                     ( $subtreeNode,
                       $interface->{$data->{'nameref'}{'ifSubtreeName'}},
-                      $param, 
+                      $ifParam, 
                       $interface->{'docsTemplates'} );
 
                 # Apply selector actions
@@ -294,6 +308,8 @@ sub buildConfig
             }
         }
     }
+
+    return;
 }
 
 
