@@ -14,7 +14,6 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Shawn Ferry <sferry at sevenspace dot com> <lalartu at obscure dot org>
 
 # Ucd Snmp Discovery
@@ -22,6 +21,8 @@
 package Torrus::DevDiscover::UcdSnmp;
 
 use strict;
+use warnings;
+
 use Torrus::Log;
 
 
@@ -135,9 +136,9 @@ sub buildConfig
     # Hostresources MIB is optional in net-snmp. We try and use the same
     # subtree name for UCD and Hostresources statistics.
     
-    my $subtreeName =
-        $devdetails->param('RFC2790_HOST_RESOURCES::sysperf-subtree-name');
-    if( not defined( $subtreeName ) )
+    my $subtreeName = $devdetails->paramString
+        ('RFC2790_HOST_RESOURCES::sysperf-subtree-name');
+    if( $subtreeName eq '' )
     {
         $subtreeName = 'System_Performance';
         $devdetails->setParam
@@ -252,6 +253,8 @@ sub buildConfig
         $cb->addLeaf( $perfNode, 'Cpu_Stats',
                       $cpuMultiParam, \@cpuMultiTemplates );
     }
+
+    return;
 }
 
 
