@@ -1,4 +1,4 @@
-#  Copyright (C) 2002  Stanislav Sinyagin
+#  Copyright (C) 2002-2011  Stanislav Sinyagin
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,17 +14,15 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Monitor;
-@Torrus::Monitor::ISA = qw(Torrus::Scheduler::PeriodicTask);
-
 use strict;
+use warnings;
+use base 'Torrus::Scheduler::PeriodicTask';
 
 use Torrus::DB;
 use Torrus::ConfigTree;
-use Torrus::Scheduler;
 use Torrus::DataAccess;
 use Torrus::TimeStamp;
 use Torrus::Log;
@@ -47,7 +45,6 @@ sub new
 
     $self->{'tree_name'} = $options{'-TreeName'};
     $self->{'sched_data'} = $options{'-SchedData'};
-    $self->{'delay'} = $options{'-Delay'} * 60;
     
     return $self;
 }
@@ -64,6 +61,7 @@ sub addTarget
         $self->{'targets'} = [];
     }
     push( @{$self->{'targets'}}, $token );
+    return;
 }
 
 
@@ -106,18 +104,24 @@ sub run
             my( $alarm, $timestamp ) = $self->$method( $config_tree, $obj );
             $obj->{'alarm'} = $alarm;
             $obj->{'timestamp'} = $timestamp;
-            
-            Debug("Monitor $mname returned ($alarm, $timestamp) ".
-                  "for token $token");
-            
-            $self->setAlarm( $config_tree, $obj );
-            undef $obj;
+
+            if( defined($alarm) )
+            {
+                Debug("Monitor $mname returned ($alarm, $timestamp) ".
+                      "for token $token");
+                $self->setAlarm( $config_tree, $obj );
+            }
+            else
+            {
+                Debug("Monitor $mname returned undefined alarm value");
+            }
         }
     }
 
     $self->cleanupExpired();
     
-    undef $self->{'db_alarms'};
+    delete $self->{'db_alarms'};
+    return;
 }
 
 
@@ -135,7 +139,6 @@ sub check_failures
     my ($value, $timestamp) = $obj->{'da'}->read_RRD_DS( $dir.'/'.$file,
                                                          'FAILURES', $ds );
     return( $value > 0 ? 1:0, $timestamp );
-
 }
 
 
@@ -221,7 +224,7 @@ sub substitute_vars
                 }
                 else
                 {
-                    $expr =~ s/\#$var/$val$1/g;
+                    $expr =~ s/\#$var/$val/g;
                 }
             }
         }
@@ -248,6 +251,7 @@ sub setAlarm
     
     my $prev_values = $self->{'db_alarms'}->get( $key );
     my ($t_set, $t_expires, $prev_status, $t_last_change);
+    $t_expires = 0;    
     if( defined($prev_values) )
     {
         Debug("Previous state found, Alarm: $alarm, ".
@@ -327,6 +331,7 @@ sub setAlarm
                                                   $t_last_change)) );
         }
     }
+    return;
 }
 
 
@@ -359,6 +364,7 @@ sub cleanupExpired
     $self->{'db_alarms'}->c_close($cursor);
     
     &Torrus::DB::checkInterrupted();
+    return;
 }
     
 
@@ -388,6 +394,7 @@ sub run_event_tset
             $config_tree->tsetDelMember($tset, $token);
         }
     }
+    return;
 }
 
 
@@ -401,7 +408,6 @@ sub run_event_exec
     my $token = $obj->{'action_token'};
     my $event = $obj->{'event'};
     my $mname = $obj->{'mname'};
-    my $timestamp = $obj->{'timestamp'};
 
     my $launch_when = $config_tree->getParam($aname, 'launch-when');
     if( not defined $launch_when )
@@ -415,6 +421,13 @@ sub run_event_exec
         $cmd =~ s/\&gt\;/\>/;
         $cmd =~ s/\&lt\;/\</;
 
+        # Make a private copy of the whole elvironment
+        local %ENV = %ENV;
+
+        # disable Perl::Critic screams for %ENV manipulation
+        # because we know what we do
+        ## no critic(Variables::RequireLocalizedPunctuationVars)
+        
         $ENV{'TORRUS_BIN'}       = $Torrus::Global::pkgbindir;
         $ENV{'TORRUS_UPTIME'}    = time() - $self->whenStarted();
 
@@ -438,7 +451,7 @@ sub run_event_exec
         $ENV{'TORRUS_EVENT'}     = $event;
         $ENV{'TORRUS_MONITOR'}   = $mname;
         $ENV{'TORRUS_MCOMMENT'}  = $config_tree->getParam($mname, 'comment');
-        $ENV{'TORRUS_TSTAMP'}    = $timestamp;
+        $ENV{'TORRUS_TSTAMP'}    = $obj->{'timestamp'};
 
         if( defined( $obj->{'display_value'} ) )
         {
@@ -510,186 +523,12 @@ sub run_event_exec
         {
             Error("$cmd executed with error: $!");
         }
-
-        # Clean up the environment
-        foreach my $envName ( keys %ENV )
-        {
-            if( $envName =~ /^TORRUS_/ )
-            {
-                delete $ENV{$envName};
-            }
-        }
     }
+    return;
 }
 
 
 
-#######  Monitor scheduler  ########
-
-package Torrus::MonitorScheduler;
-@Torrus::MonitorScheduler::ISA = qw(Torrus::Scheduler);
-
-use Torrus::ConfigTree;
-use Torrus::Log;
-use Torrus::Scheduler;
-use Torrus::TimeStamp;
-
-sub beforeRun
-{
-    my $self = shift;
-
-    my $tree = $self->treeName();
-    my $config_tree = new Torrus::ConfigTree(-TreeName => $tree, -Wait => 1);
-    if( not defined( $config_tree ) )
-    {
-        return undef;
-    }
-
-    my $data = $self->data();
-
-    # Prepare the list of tokens, sorted by period and offset,
-    # from config tree or from cache.
-
-    my $need_new_tasks = 0;
-
-    Torrus::TimeStamp::init();
-    my $known_ts = Torrus::TimeStamp::get($tree . ':monitor_cache');
-    my $actual_ts = $config_tree->getTimestamp();
-    if( $actual_ts >= $known_ts )
-    {
-        if( $self->{'delay'} > 0 )
-        {
-            Info(sprintf('Delaying for %d seconds', $self->{'delay'}));
-            sleep( $self->{'delay'} );
-        }
-
-        Info("Rebuilding monitor cache");
-        Debug("Config TS: $actual_ts, Monitor TS: $known_ts");
-
-        undef $data->{'targets'};
-        $need_new_tasks = 1;
-
-        $data->{'db_tokens'} = new Torrus::DB( 'monitor_tokens',
-                                               -Subdir => $tree,
-                                               -WriteAccess => 1,
-                                               -Truncate    => 1 );
-        $self->cacheMonitors( $config_tree, $config_tree->token('/') );
-        # explicitly close, since we don't need it often, and sometimes
-        # open it in read-only mode
-        $data->{'db_tokens'}->closeNow();
-        undef $data->{'db_tokens'};
-
-        # Set the timestamp
-        &Torrus::TimeStamp::setNow($tree . ':monitor_cache');
-    }
-    Torrus::TimeStamp::release();
-
-    &Torrus::DB::checkInterrupted();
-
-    if( not $need_new_tasks and not defined $data->{'targets'} )
-    {
-        $need_new_tasks = 1;
-
-        $data->{'db_tokens'} = new Torrus::DB('monitor_tokens',
-                                              -Subdir => $tree);
-        my $cursor = $data->{'db_tokens'}->cursor();
-        while( my ($token, $schedule) = $data->{'db_tokens'}->next($cursor) )
-        {
-            my ($period, $offset, $mlist) = split(':', $schedule);
-            if( not exists( $data->{'targets'}{$period}{$offset} ) )
-            {
-                $data->{'targets'}{$period}{$offset} = [];
-            }
-            push( @{$data->{'targets'}{$period}{$offset}}, $token );
-            $data->{'mlist'}{$token} = [];
-            push( @{$data->{'mlist'}{$token}}, split(',', $mlist) );
-        }
-        $data->{'db_tokens'}->c_close($cursor);
-        undef $cursor;
-        $data->{'db_tokens'}->closeNow();
-        undef $data->{'db_tokens'};
-    }
-
-    &Torrus::DB::checkInterrupted();
-
-    # Now fill in Scheduler's task list, if needed
-
-    if( $need_new_tasks )
-    {
-        Verbose("Initializing tasks");
-        my $init_start = time();
-        $self->flushTasks();
-
-        foreach my $period ( keys %{$data->{'targets'}} )
-        {
-            foreach my $offset ( keys %{$data->{'targets'}{$period}} )
-            {
-                my $monitor = new Torrus::Monitor( -Period => $period,
-                                                   -Offset => $offset,
-                                                   -TreeName => $tree,
-                                                   -SchedData => $data );
-
-                foreach my $token ( @{$data->{'targets'}{$period}{$offset}} )
-                {
-                    &Torrus::DB::checkInterrupted();
-                    
-                    $monitor->addTarget( $config_tree, $token );
-                }
-
-                $self->addTask( $monitor );
-            }
-        }
-        Verbose(sprintf("Tasks initialization finished in %d seconds",
-                        time() - $init_start));
-    }
-
-    Verbose("Monitor initialized");
-
-    return 1;
-}
-
-
-sub cacheMonitors
-{
-    my $self = shift;
-    my $config_tree = shift;
-    my $ptoken = shift;
-
-    my $data = $self->data();
-
-    foreach my $ctoken ( $config_tree->getChildren( $ptoken ) )
-    {
-        &Torrus::DB::checkInterrupted();
-
-        if( $config_tree->isSubtree( $ctoken ) )
-        {
-            $self->cacheMonitors( $config_tree, $ctoken );
-        }
-        elsif( $config_tree->isLeaf( $ctoken ) and
-               ( $config_tree->getNodeParam($ctoken, 'ds-type') ne
-                 'rrd-multigraph') )
-        {
-            my $mlist = $config_tree->getNodeParam( $ctoken, 'monitor' );
-            if( defined $mlist )
-            {
-                my $period = sprintf('%d',
-                                     $config_tree->getNodeParam
-                                     ( $ctoken, 'monitor-period' ) );
-                my $offset = sprintf('%d',
-                                     $config_tree->getNodeParam
-                                     ( $ctoken, 'monitor-timeoffset' ) );
-                
-                $data->{'db_tokens'}->put( $ctoken,
-                                           $period.':'.$offset.':'.$mlist );
-                
-                push( @{$data->{'targets'}{$period}{$offset}}, $ctoken );
-                $data->{'mlist'}{$ctoken} = [];
-                push( @{$data->{'mlist'}{$ctoken}}, split(',', $mlist) );
-            }
-        }
-    }
-}
-
 
 1;
 
