@@ -17,8 +17,9 @@
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Renderer;
-
 use strict;
+use warnings;
+
 use Digest::MD5 qw(md5_hex);
 
 use Torrus::DB;
@@ -207,6 +208,7 @@ sub setCache
     $self->{'db'}->put( $keystring,
                         join(':',
                              ($t_render, $t_expires, $filename, $mime_type)));
+    return;
 }
 
 
@@ -228,6 +230,7 @@ sub checkAndClearCache
         Torrus::TimeStamp::setNow($tree . ':renderer_cache');
     }
     Torrus::TimeStamp::release();
+    return;
 }
 
 
@@ -247,6 +250,7 @@ sub clearcache
     }
     $self->{'db'}->c_close($cursor);
     Debug('Renderer cache cleared');
+    return;
 }
 
 
