@@ -14,12 +14,11 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Renderer::Frontpage;
-
 use strict;
+use warnings;
 
 use Torrus::ConfigTree;
 use Torrus::Search;
@@ -266,10 +265,10 @@ sub doGlobalSearch
     
     foreach my $element ( @{$sorted} )
     {
-        my $string = join( ':', $element->[0], $element->[1] );
-        if( not $seen{$string} )
+        my $str = join( ':', $element->[0], $element->[1] );
+        if( not $seen{$str} )
         {
-            $seen{$string} = 1;
+            $seen{$str} = 1;
             push( @{$ret}, $element );
         }
     }
