@@ -36,7 +36,8 @@ my %rrd_graph_opts =
      'width'  => '--width',
      'height' => '--height',
      'imgformat' => '--imgformat', 
-     );
+     'border' => '--border',
+    );
 
 my %mime_type =
     ('PNG' => 'image/png',
@@ -506,7 +507,7 @@ sub rrd_make_multigraph
             push( @{$obj->{'args'}{'line'}},
                   sprintf( '%s:%s%s%s%s', $linestyle, $dname,
                            $linecolor,
-                           ($legend ne '') ? ':'.$legend.'\l' : '',
+                           ($legend ne '') ? ':'.$legend.'\l' : ':\l',
                            $stack ) );
         }
     }
@@ -661,6 +662,11 @@ sub rrd_make_graphline
     push( @{$obj->{'args'}{'line'}},
           sprintf( '%s:%s%s%s', $linestyle, $obj->{'dname'}, $linecolor,
                    ($legend ne '') ? ':'.$legend.'\l' : '' ) );
+    if( $legend eq '' )
+    {
+        push( @{$obj->{'args'}{'line'}}, 'COMMENT:\l' );
+    }
+    
     return;
 }
 
@@ -688,7 +694,7 @@ sub rrd_make_maxline
 
     if( not defined( $legend ) )
     {
-        $legend = '';
+        $legend = 'Max';
     }
     else
     {
@@ -719,7 +725,7 @@ sub rrd_make_maxline
 
     push( @{$obj->{'args'}{'line'}},
           sprintf( '%s:%s%s%s', $linestyle, $max_dname, $linecolor,
-                   ($legend ne '') ? ':'.$legend.'\l' : '' ) );
+                   ($legend ne '') ? ':'.$legend.'\l' : ':\l' ) );
     return;
 }
 
@@ -967,6 +973,48 @@ sub rrd_make_graph_opts
         }
     }
 
+    # take colors from view and URL params
+    my $colorval =
+        $self->{'options'}->{'variables'}->{'Gcolors'};
+    
+    if( not defined( $colorval ) )
+    {
+        $colorval = $config_tree->getParam( $view, 'graph-colors' );
+    }
+        
+    if( defined( $colorval ) )
+    {
+        my @values = split( /:/, $colorval );
+        if( (scalar(@values) % 2) != 0 )
+        {
+            Error("Graph colors should be an even number of " .
+                  "elements separated by colon: " . $colorval);
+        }
+        else
+        {
+            while( scalar(@values) )
+            {
+                my $tag = shift @values;
+                my $color = shift @values;
+                if( $tag !~ /^[A-Z]+$/o )
+                {
+                    Error("Invalid format for color tag in graph colors: " .
+                          $colorval);
+                }
+                elsif( $color !~ /^[0-9A-F]{6}([0-9A-F]{2})?$/io )
+                {
+                    Error("Invalid format for color value in graph colors: " .
+                          $colorval);
+                }
+                else
+                {
+                    push( @args, '--color', $tag . '#' . $color );
+                }
+            }
+        }
+        
+    }
+
     if( scalar( @Torrus::Renderer::graphExtraArgs ) > 0 )
     {
         push( @args, @Torrus::Renderer::graphExtraArgs );
