@@ -1,4 +1,4 @@
-#  Copyright (C) 2002  Stanislav Sinyagin
+#  Copyright (C) 2002-2011  Stanislav Sinyagin
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,24 +14,23 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 
-# Task scheduler runtime information. Quite basic statistics access.
+# Search database interface
 
 package Torrus::Search;
+use strict;
+use warnings;
 
 use Torrus::DB;
 use Torrus::Log;
-use strict;
-
 
 sub new
 {
-    my $self = {};
     my $class = shift;
     my %options = @_;
+    my $self = {};
     bless $self, $class;
 
     %{$self->{'options'}} = %options;
@@ -54,6 +53,7 @@ sub openTree
           -Truncate => $self->{'options'}{'-WriteAccess'} );
 
     $self->{'db_treewords'}{$tree} = $db;
+    return;
 }
 
 
@@ -63,6 +63,7 @@ sub closeTree
     my $tree = shift;
 
     $self->{'db_treewords'}{$tree}->closeNow();
+    return;
 }
 
 
@@ -78,6 +79,7 @@ sub openGlobal
           -Truncate => $self->{'options'}{'-WriteAccess'} );
 
     $self->{'db_globwords'} = $db;    
+    return;
 }
 
 
@@ -106,6 +108,7 @@ sub storeKeyword
 
         $self->{'stored'}{$lookupkey} = 1;
     }
+    return;
 }
 
 sub searchPrefix
